//============================================================================
// FILE: SwigFunction.h
//
// TYPE: C-LIBRARY
// DESCRIPTION: 
// COMMENT: Ce fichier n'est pris en charge que par makeCompileSWIG
// 
// AUTHORS: Rellier/Remion (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _SWIG_HH
#define _SWIG_HH

// SYSTEM INCLUDES
//


// PROJECT INCLUDES
//
#include "Method.h"

/**@name CDPFORTEXExternalisation de donn\'ees fonctionnelles */
//@{ 

/**@name G\'en\'eralit\'es
\label{externFonc}Une donnée fonctionnelle est une connaissance sur le système qui s'exprime, non pas par une valeur numérique ou symbolique, scalaire ou vectorielle, donnée à un attribut du système, mais par une fonction. Une fonction renvoie une valeur (scalaire ou vectorielle) et/ou réalise un effet de bord sur certains attributs du système. Ce que fait la fonction est déterminé par l'état du système et de son environnement au moment de l'invocation.\\

Le développeur d'un simulateur sous \Db peut coder une connaissance fonctionnelle, par exemple pour une entité, de deux manières différentes :
\begin{itemize}
\item Créer une sous-classe de [C]Method[/C], puis en attacher une instance à l'entité par [C]AddMethod[/C]. La sous-classe peut être dotée d'un corps dans le constructeur, mais ce corps peut être ensuite "surchargé" (remplacé) par un autre (par [Y]AssignBody[/Y]). Dans les deux cas, le corps est une fonction C++ classique déclarée et réalisée dans les fichiers de l'application (par exemple [F]UserEvent.cc, *.h[/F] pour les événements).\\
Ce n'est pas le cas de la classe [C]Entity[/C], mais on sait que certaines classes de base de \Db ([C]Event[/C], par exemple) sont dotées de méthodes prédéfinies : \Db impose leur identité (symbole et nom de classe) et les affecte automatiquement aux instances des classes. Pour leur adjoindre une mé\-thode non prédéfinie, le développeur doit en coder lui-même le constructeur (et par là-même décider de l'identité de la méthode), puis invoquer explicitement [Y]AddMethod[/Y] dans le constructeur de la classe dont la méthode est un attribut fonctionnel.

\item Doter l'entité, pour suivre cet exemple, d'une fonction membre, au sens des langages à objets. La fonction est déclarée dans la partie publique ou privée de la classe et réalisée par ailleurs. Si le membre déclaré possède le type 'pointeur sur une fonction' (appelons-le {small {\tt (* m)}}), alors le développeur peut lui affecter comme valeur une fonction (disons {small {\tt f}}) écrite par ailleurs (par {small {\tt m = f}}).
\end{itemize}

Dans les deux cas, le corps de fonction codant la connaissance fonctionnelle est compilé puis intégré dans le module exécutable (le simulateur). Il est clair que toute modification du code passe par les étapes de compilation et d'intégration à l'exécutable (édition des liens). \\

Cette section expose les procédures d'externalisation de données fonctionnelles. L'externalisation, c'est le placement du corps des fonctions correspondantes à l'extérieur du module exécutable et l'établissement d'un pont entre les deux. Par ce pont transitent des commandes (l'exécutable invoque une fonction externe et la fonction invoque des services de \D) et des données (l'exécutable passe des arguments à la fonction externe et la fonction lui renvoie un résultat ou modifie par effet de bord la valeur de variables dans l'espace d'adressage de l'exécutable). Il est important de noter deux choses :
\begin{itemize}
\item Seules les connaissances codées par des instances de la classe [C]Method[/C] peuvent être externalisées, à l'exclusion de celles codées par des fonctions membres.
\item Le code externalisé n'est pas compilé, c'est-à-dire qu'il est pris en charge par un interpréteur. La présente version de \Db supporte des connaissances externalisées codées en {\em perl}.
\end{itemize}

Dans la suite, on indique d'abord que les procédures d'externalisation requièrent une version de \Db appropriée. Puis on décrit ce qu'il faut changer dans le code d'un simulateur existant pour externaliser une fonction. On donne au passage quelques règles d'écriture de la version {\em perl} de la fonction. Enfin, on décrit ce qu'il faut faire pour que ce code {\em perl} puisse intégrer des appels à des services de \D. Dans ce dernier point intervient l'outil SWIG\index{SWIG}. 
 */ 

/**@name L'externalisatiion requiert une version de \Db appropri\'ee
L'externalisation de fonctions requiert d'installer une bibliothèque DIESE spécialement équipée. Pour cela il suffit d'avoir compilé la bibliothèque en donnant la valeur {\em 1} au symbole {small {\tt \_SWIG\_}}. A cet effet, le fichier de directives [F]compileModulesSWIG[/F] fourni avec DIESE pour une compilation sous Unix/Linux, comprend une commande :
\begin{verbatim}
[P]g++ -D\_SWIG\_[/P]
\end{verbatim}

Il faut ensuite générer le simulateur avec la même option de compilation, et réaliser l'édition des liens avec la bibliothèque \Db adaptée à l'externalisation. Un fichier de directives [F]makeSimulatorSWIG[/F] doit être fourni à cet effet avec chaque simulateur, par le développeur.\\

Au-delà de cette contrainte sur la génération du simulateur exécutable, il existe des contraintes sur le codage des fichiers 'sources' du simulateur, précisées dans une des sections suivantes.
 */

/**@name Les \'etapes de l'externalisation d'une fonction
Dans cette section, on considèrera pour exemple la fonction [Y]SetCommand\_Exec[/Y] qui code dans le fichier [F]UserDiscreteProcess.cc[/F] la détermination de la commande (freinage / accélération) dans l'application du {\em toboggan} (voir la page 'Présentation générale et exemples')). Elle est affectée par [Y]SetExec[/Y] au corps de l'instance de la mé\-thode prédéfinie [Y]ExecProcess[/Y] attachée au processus [C]SetCommand[/Y] lors de la construction de celui-ci.\\[2mm]

{\large {\tt Ecrire la routine {\em perl}}} \\

... et la placer dans un fichier [F]<nom\_fichier>.pl[/F] dont on dira plus loin comment il est exploité par le simulateur. L'ossature du code est la suivante :
\begin{verbatim}
sub freinageAcceleration_execProcess {
    $pM = @_[0];
    ...
}
\end{verbatim}
Le code complet sera donné dans la section qui indique la manière d'appeler des services de \Db dans le code {\em perl}.\\

Noter cependant ici que la routine {\em perl} reçoit de \Db en argument implicite le pointeur sur l'instance de [C]Method[/C] qui est l'argument explicite de la fonction [Y]SetCommand\_Exec[/Y]. Le langage {\em perl} référence cet argument par {\small {\tt @\_[0]}}. On affecte cette valeur à la variable locale {\small {\tt \$pM}} (noter que {\em perl} préfixe toute variable par le caractère 'dollar').\ \

Noter encore la règle de nommage de la routine {\em perl}. Alors que le nom de la fonction C++ est libre (le nom choisi devient l'argument de la méthode [Y]SetExec[/Y] dans le constructeur du processus), le nom de la routine doit être d'une des deux formes suivantes :
\begin{itemize}
\item pour les méthodes prédéfinies : la concaténation du nom de classe de l'objet (processus, événement, \ldots), du signe '\_' (code ascii 95), du nom de classe de la méthode et enfin de la chaîne {\small {\tt \_Body}}.
\item pour les méthodes non prédéfinies : la concaténation du nom de classe de la méthode et de la chaîne {\small {\tt \_Body}}.
\end{itemize}

[F]{\large Doter le simulateur d'un interpréteur {\em perl[/F]}} \\

La routine {\em perl}, bien qu'écrite, n'est pas d'une utilisation obligatoire. Si un interpréteur {\em perl} n'a pas été créé, ou s'il a été créé en lui donnant l'adresse d'un autre fichier que celui dans lequel on a rangé la routine, alors \Db continue d'invoquer la version compilée ([Y]SetCommand\_Exec[/Y]).\\

Pour créer un interpréteur, il suffit d'invoquer le service \Db [Y]NewPerlInterpreter[/Y].
\begin{itemize}
\item soit explicitement dans le programme principal, normalement en son début, et en tous cas avant ce qui aurait été la première invocation de [Y]SetCommand\_Exec[/Y],
\item soit implicitement et automatiquement, en utilisant l'option PERL\_FIC dans la phrase INITIALISATION SIMULATION du fichier des spécifications de la simulation (voir page 'Le fichier des spécifications de simulation'). Ce qui s'écrit ainsi :
\begin{verbatim}
INITIALISATION SIMULATION
     ....  // autres options
     PERL_FIC "../perl/toboggan.pl";  // contient la routine perl
;
\end{verbatim}
\end{itemize}
Attention ! La fonction [Y]NewPerlInterpreter[/Y] associe un fichier à l'interpréteur. Ce fichier est bien sûr celui mentionné après le mot-clé PERL\_FIC dans la phrase ci-dessus. L'invocation explicite dans le programme principal doit être précédée de la désignation de ce fichier. Cela se fait par l'instruction suivante : 
\begin{verbatim}
     pCurrentSim->SetPerlFile("../perl/toboggan.pl");
\end{verbatim}
La version courante de \Db ne sait gérer qu'un seul interpréteur à la fois, et cet interpréteur ne peut accéder qu'à un seul fichier à la fois. Cependant, il est possible de diriger l'interpréteur sur plusieurs fichiers successivement : dans \D, il faut le supprimer (voir plus loin) puis le créer à nouveau en l'associant à un fichier différent.\\[2mm]

{\large {\tt Exploiter les services \Db d'interprétation de routines {\em perl}}} \\

Pour notre exemple, ceci est réalisé par l'appel suivant : 
\begin{verbatim}
     PerlVoidFunction(str, pM);
\end{verbatim}
où :\\
\indent {\small {\tt str}} est le nom de la routine {\em perl}. Ici : {\small {\tt "freinageAcceleration\_execProcess"}}\ \
\indent {\small {\tt pM}} est l'instance de [C]Method[/C] qui sera passée à la routine {\em perl} en argument implicite de rang 0, autrement dit l'instance de [C]Method[/C] (du processus) dont le corps est externalisé. Ici, c'est l'instance de la classe prédéfinie [C]ExecProcess[/C] qui a reçu comme corps compilé la fonction [Y]SetCommand\_Exec[/Y].\ \

							L'appel ci-dessus est construit et réalisé automatiquement par \D, plus précisément par les services tels que [Y]ExecVoidMethod[/Y] de la classe [C]Process[/C], lequel est justement celui qui est invoqué dans notre exemple, puisque la méthode prédéfinie en question possède un corps de type {\em void} (voir la classe [C]Process[/C], page 'Affectation d'un corps aux méthodes', et 'Méthodes prédéfinies de processus/Exécution d'un processus discret').\ \

Quant à l'appel à [Y]ExecVoidMethod[/Y], il est fait lui aussi automatiquement dans la méthode du moteur de simulation qui réalise l'exécution de l'effet d'un processus ponctuel, lors du dépilement de l'événement correspondant. En fait, l'exploitation des services d'interprétation de routines {\em perl} est entièrement pris en charge par \Db pour les méthodes prédéfinies. Par contre, le développeur doit invoquer lui-même les services tels que [Y]ExecVoidMethod[/Y] pour les méthodes non prédéfinies.\\[2mm]

{\large {\tt Supprimer l'interpréteur {\em perl}}} \\

Lorsque plus aucun appel à l'interpréteur {\em perl} ne doit être fait, il est bon de le supprimer, par une invocation de [Y]DeletePerlInterpreter[/Y]. Un tel appel est fait automatiquement dans le destructeur de la simulation courante ([Y]pCurrentSim[/Y]) si un interpréteur a été créé. On sait qu'on libère la mémoire occupée par [Y]pCurrentSim[/Y] par un appel explicite [P]delete pCurrentSim;[/P] dans le programme principal, ou bien, dans un fichier de directives de la simulation, par une directive DELETE SIMULATION ou DELETE ALL.
 */

/**@name L'appel des services de \Db dans le code externalis\'e
Les fonctions (corps de méthodes) externalisées font généralement appel à des services de \D. Parmi les plus fréquemment utilisés figurent les services des sous-classes de [C]Method[/C] qui renvoient l'objet (entité, processus, \ldots) dont l'instance de [C]Method[/C] est un attribut fonctionnel. Dans notre exemple, on utilise le service [Y]DescribedDiscreteProcess[/Y] de la classe [C]ProcessMethod[/C]. Y figurent aussi les services de la classe [C]Method[/C] qui renvoient les arguments (instances d'[C]Argument[/C]) de la méthode. Il se trouve que la méthode choisie pour notre exemple ([Y]ExecProcess[/Y]) ne possède pas d'argument.\\

Voici le code de la routine {\em perl} réalisant le même effet que la fonction [Y]SetCommand\_Exec[/Y] :
\begin{verbatim} 
use bdiese;
use appli;
sub freinageAcceleration_execProcess {
    $pM = @_[0];
    $pP = bdiese::ProcessMethod_DescribedDiscreteProcess($pM);
    $pDriver = bdiese::Process_ProcessedEntity($pP);
    $pCar = bdiese::Entity_GetSuperEntity($pDriver);
    $X = bdiese::Entity_GetFloatValue($pCar, $appli::POSITION);
    $V = bdiese::Entity_GetFloatValue($pCar, $appli::SPEED);
    $frontiere = - 0.06*$X**2 - 0.03*$X + 0.01;
    if($V < $frontiere) {
       bdiese::Entity_SetValue($pDriver, $bdiese::COMMAND, -1);
    }
    else {
       bdiese::Entity_SetValue($pDriver, $bdiese::COMMAND, +1);
    }
}
\end{verbatim}
On a de fait illustré les quelques règles de transcription du code C++, en général préexistant, en code {\em perl} :
\begin{itemize}
\item tous les objets (variables, classes, méthodes, éléments d'énumération, \ldots) de l'espace d'adressage C++ sont préfixés par le nom d'un {\em module} concaténé avec la chaîne {\small {\tt ::}}. Le nom de ce module est soit {\small {\tt bdiese}} si l'objet est déclaré dans la bibliothèque \D, soit un autre nom si l'objet est déclaré dans le code propre du simulateur ; on dira plus loin la raison de cela et comment ce dernier nom est attribué,

\item les méthodes sont désignées par un identificateur construit par concaténation du nom de la classe C++ dans laquelle la méthode est déclarée, du signe '\_' (code ascii 95) et du nom de la méthode (attention ! il s'agit du nom C++ et pas du nom de classe). 

\item toute méthode ainsi désignée prend en premier argument explicite l'objet auquel elle est appliquée. Autrement dit : la variable {\em perl} correspondant à l'objet C++ auquel est appliquée la méthode dans le code C++.
\item on rappelle enfin que l'identificateur d'une variable {\em perl} débute toujours par le caractère {\small {\tt \$}}.
\end{itemize}

Reste à dire qu'au-delà du respect de ces règles d'écriture, il existe une autre condition pour que l'interpréteur {\em perl} sache reconnaître et manipuler les objets et services de \Db : on doit avoir intégré dans le code exécutable du simulateur des "versions" des objets C++ spécialement "équipées" à cet effet. Cet "équipement" a lieu dans les procédures de compilation et d'édition de liens, celle de la bibliothèque \Db pour les services qu'elles fournit, ou bien celle du simulateur pour les autres objets et services. Les objets équipés sont placés dans des librairies (les {\em modules}) qui faut charger en préambule du fichier des routines, à l'aide de la directive [P]use <module>[/P]. La section suivante expose comment satisfaire cette condition.
 */

/**@name La pr\'eparation des objets C++ \`a leur int\'egration dans du code {\em perl}
C'est l'outil SWIG (Simplified Wrapper and Interface Generator) qui est utilisé pour faire cette préparation. Aussi bien pour la bibliothèque \Db que pour le simulateur lui-même, elle se déroule selon les quatre étapes suivantes.\\[2mm]

{\large {\tt Installer le système de répertoires}} \\

Soit {\small {\tt <rep>}} le répertoire d'accueil : c'est {\small {\tt bdiese/}} pour \Db et {\small {\tt toboggan/}} pour le simulateur du toboggan. On a créé dans {\small {\tt <rep>}} un répertoire pour les fichiers sources ({\small {\tt source}}). Celui sous {\small {\tt bdiese/}} comprend un fichier de directives de compilation spécial [F]compileModuleSWIG[/F]. Celui sous {\small {\tt toboggan}} comprend un fichier de directives [F]makeSimulatorSWIG[/F].\\

On crée sous {\small {\tt rep}} un répertoire {\small {\tt swig}} ainsi structuré :
\begin{verbatim}
rep/
   source/
   swig/
      incl/
      doc/
\end{verbatim}

{\large {\tt Ecrire les fichiers d'informations pour SWIG}} \\
On crée dans {\small {\tt incl/}} un fichier de suffixe {\small {\tt .i}} ({\small {\tt i}} pour 'interface') pour chaque fichier d'entêtes (dans {\small {\tt source}}) qui mentionne au moins un objet ou service qu'on souhaite équiper pour être reconnu par {\em perl}. En principe, le fichier débute par la déclaration d'un {\em module perl} dans lequel seront regroupés les objets et services équipés. Il se poursuit par les déclarations des objets à équiper. La déclaration se fait à l'identique du fichier d'entêtes pour les variables, structures et types énumérés. Pour les classes, on reprend la déclaration du fichier d'entêtes en excluant éventuellement les membres privés ou protégés et les services qu'on ne souhaite pas équiper. Il est important de noter que, pour chaque classe, un seul constructeur peut être déclaré dans le fichier [F].i[/F].\\

En réalité, le répertoire {\small <rep>/swig/incl/{\small  contient un fichier ([F]bdiese.i[/F] et, pour l'application du toboggan, [F]appli.i[/F]) qui réalise l'inclusion de tous les fichiers [F].i[/F] correspondant aux fichiers [F].h[/F]. Ainsi, le contenu de [F]bdiese.i[/F] est il le suivant :
\begin{verbatim}
%module bdiese

%{
#include "CoreTypes.h"
#include "Entity.h"
...
%}

%include "../swig/incl/CoreTypes.i"
%include "../swig/incl/Entity.i"
...
\end{verbatim}

Voici alors un extrait du fichier [F]CoreTypes.i[/F]. Noter l'absence de ligne {[P]\%module[/P] : 
\begin{verbatim}
extern Simulation* pCurrentSim;
typedef int UserDescriptorId;
enum ObjectTypeId {BD_BASIC_ENTITY, BD_DESCRIPTED_ENTITY, BD_ENTITY, BD_ENTITYSPEC, BD_EVENT, BD_MONITOR, BD_PROCESS};
...
\end{verbatim}
... et un extrait du fichier [F]Entity.i[/F] : 
\begin{verbatim}
class Entity
{
 public:
  int         GetIntValue(UserDescriptorId);
  float       GetFloatValue(UserDescriptorId);
  char*       InstanceName() const;
  ...
};
...
\end{verbatim}

Pour le simulateur du toboggan, le fichier [F]appli.i[/F] est celui-ci : 
\begin{verbatim}
%module appli

%{
#include "../source/UserTypes.h" 
#include "../source/UserEntity.h"
  ...
%}

%include "../swig/incl/UserTypes.i"
%include "../swig/incl/UserEntity.i"
  ...
\end{verbatim}

{\large {\tt Réaliser avec SWIG les versions intégrables dans {\em perl}}} \\

Il suffit d'invoquer la commande système [P]swig[/P] en indiquant le langage intégrateur ({\em perl}), le type de langage à intégrer ({\em C++)} et le fichier des interfaces. Par exemple, pour intégrer les objets du {\em toboggan} :
\begin{verbatim}
swig -perl -C++ -dlatex appli.i
\end{verbatim}
Cela génère automatiquement une documentation au format demandé (option {\small {\tt -d}}, dans {\small {\tt doc/}}) et un programme C qui doit être compilé en indiquant le chemin d'accès à la bibliothèque {\em perl}. L'étape suivante est la génération d'une librairie dynamique qui regroupe le fichier objet issu de cette compilation et les modules objets issus de la compilation des fichiers source C++. La confection de la librairie \Db s'arrête là. La confection d'un simulateur nécessite en plus l'assemblage dans un programme exécutable (par édition des liens) du module objet du programme principal, de la librairie dynamique \Db (créée à partir de [F]bdiese.i[/F]) et de la librairie dynamique de l'application (créée à partir de [F]appli.i[/F]). \\

Une procédure qui enchaîne toutes ces opérations est fournie dans les fichiers [F]compileModuleSWIG[/F] et [F]makeSimulatorSWIG[/F].\\[2mm]

{\large {\tt Complément sur la création de l'interpréteur {\em perl}}} \\

Pour que l'interpréteur sache reconnaître et manipuler les objets C++ équipés, la fonction de création de l'interpréteur doit comporter une option spéciale, sous la forme d'une fonction de nom [Y]xs\_init[/Y] pour permettre à la directive  [P]use <module>[/P] de fonctionner correctement. Si on ne faisait jamais appel à des objets C++ dans le code {\em perl}, on utiliserait le pointeur de fonction {\small {\tt NULL}} en lieu et place de [Y]xs\_init[/Y] dans le code de [Y]NewInterpreter[/Y]. La fonction [Y]xs\_init[/Y] est déclarée dans [F]Swig.h[/F] (voir ci-dessous) pour être référencée dans [Y]NewInterpreter[/Y]. 
\noLF
Elle est initialisée au pointeur {\small {\tt NULL}} dans [F]Swig.cc[/F]. Ce qui revient alors au développeur d'une application, c'est de surcharger la valeur de [Y]xs\_init[/Y], en en plaçant une version appropriée de nom [Y]App\_xs\_init[/Y] dans le code du simulateur, par exemple dans les fichiers [F]UserTypes.*[/F] et en réalisant, typiquement dans le programme principal, l'affectation suivante entre fonctions :
\begin{verbatim}
      xs_init = App_xs_init;
\end{verbatim}
[Y]App\_xs\_init[/Y] comprend une ligne [P]EXTERN\_C[/P] et une ligne [P]newXS[/P] pour chaque librairie dynamique incluse dans le simulateur. Une version standard en est la suivante :
\begin{verbatim}
#include <EXTERN.h>
#include <perl.h>

EXTERN_C void boot_bdiese _((CV* cv));
EXTERN_C void boot_appli _((CV* cv));  

void App_xs_init() {
  char* file = __FILE__;
  newXS("bdiese::bootstrap", boot_bdiese, file);
  newXS("appli::bootstrap", boot_appli, file);
}
\end{verbatim}
 */ 


/**@name Fonctions de gestion de l'interpr\'eteur {\em perl} */
//@{ 
/** Création de l'interpréteur.\index{fonctions!NewPerlInterpreter} */
void NewPerlInterpreter();
/** Suppression de l'interpréteur, seulement si on en a créé un.\index{fonctions!DeletePerlInterpreter} */
void DeletePerlInterpreter();
/** Renvoie {\em faux} si un interpréteur {\em perl} n'a pas été créé, ou s'il a été créé en lui donnant l'adresse d'un autre fichier que celui dans lequel on a rangé la routine, ou si la routine n'est pas présente dans le fichier associé à l'interpréteur.
Fonction exploitée exclusivement par les méthodes telles que [Y]ExecVoidMethod[/Y] pour se diriger vers l'interprétation de la routine {\em perl} dont le nom est passé en paramètre, si elle existe, ou bien l'exécution de la version compilée. */
bool IsPerlExtern(char*); 
     
/** Désigne le chemin d'accès au fichier des routines {\em perl}. 
Le chemin est exprimé par rapport au répertoire de lancement de la commande d'exécution, ou bien absolu.\index{méthodes!SetPerlFile} */
  void               SetPerlFile(char*); 
  /** Retourne le chemin d'accès au fichier des routines {\em perl}, ou une chaîne vide s'il n'en existe pas.\index{méthodes!GetPerlFile} */
  char*              GetPerlFile(); 
  /** Fonction surchargeable à l'usage de [Y]NewInterpreter[/Y] (voir en fin de la section précédente). */
  extern void (*xs_init)();
//@}  


/**@name Fonctions d'interpr\'etation de routines {\em perl}
On rappelle que la fonction [Y]PerlVoidFonction[/Y] est exploitée par le service [Y]ExecVoidMethod[/Y]. De même [Y]PerlPredicate[/Y] est exploitée par [Y]ExecPredicate[/Y], etc.. Toutes ces fonctions prennent comme premier argument le nom de la routine {\em perl} à interpréter. Le second argument est un pointeur sur une instance de [C]Method[/C], en fait de [C]EntityMethod[/C] si la routine est une connaissances fonctionnelle d'une entité, etc.
*/
//@{ 
  /**  */
void PerlVoidFunction(char*, EntityMethod*);
         /// 
bool PerlPredicate(char*, EntityMethod* );
         /// 
int PerlIntFunction(char*, EntityMethod*);
         /// 
float PerlFloatFunction(char*, EntityMethod*);
         /// 
double PerlDoubleFunction(char*, EntityMethod*);
         /// 
BasicEntity* PerlEntityFunction(char*, EntityMethod*);
         /// 
char* PerlStringFunction(char*, EntityMethod*);
         /// 

         /**  */
void PerlVoidFunction(char*, ProcessMethod*);
         /// 
int PerlIntFunction(char*, ProcessMethod*);
         /// 

         /**  */
void PerlVoidFunction(char*, EventMethod*);
         /// 
bool PerlPredicate(char*, EventMethod* );
         /// 
int PerlIntFunction(char*, EventMethod*);
         /// 

         /**  */
void PerlVoidFunction(char*, MonitorMethod*);
         /// 
int PerlIntFunction(char*, MonitorMethod*);
         /// 
float PerlFloatFunction(char*, MonitorMethod*);
         /// 
double PerlDoubleFunction(char*, MonitorMethod*);
         /// 
BasicEntity* PerlEntityFunction(char*, MonitorMethod*);
         /// 
pEntityTab* PerlEntityTabFunction(char*, MonitorMethod*);
         /// 
char* PerlStringFunction(char*, MonitorMethod*);
         /// 
DescValueSpec* PerlDescValueSpecFunction(char*, MonitorMethod*);
         /// 
pDescValueSpecTab* PerlDescValueSpecTabFunction(char*, MonitorMethod*);
         /// 

         /**  */
bool PerlPredicate(char*, EntitySpecMethod* );
         /// 
int PerlIntFunction(char*, EntitySpecMethod*);
         /// 
BasicEntity* PerlEntityFunction(char*, EntitySpecMethod*);
         /// 
pEntityTab* PerlEntityTabFunction(char*, EntitySpecMethod*);

//@}  // fin fonctions

//@} // fin externalisation

#endif  // _SWIG_HH
