//============================================================================
// FILE: C_Activity.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête de la classe générique des activités (C-DIESE)
// 
// AUTHORS: Baali/Rellier (INRA/UBIA, Toulouse)
// REFERENCE : "Fondements ontologiques des systèmes pilotés"
//             R. Martin-Clouaire, J.-P. Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _C_ACTIVITY_HH
#define _C_ACTIVITY_HH

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//
#include "C_CoreTypes.h"
      
 

// fermeture dans nonprimitive
/**@name Les activités 
\begin{itemize} \item Directive d'inclusion : 
[P]\#include C_Activity.h
[/P]
\end{itemize}

{\large  {\tt Définitions}}
 
\begin{itemize} 
\item Activité\\
Les activités sont des entités qui définissent les opérations qu'il convient de réaliser sur le système pour satisfaire l'objectif de la personne dotée de la stratégie (classe [C]Manager[/C], voir la page dédiée aux systèmes). Elles composent le {\em plan d'activités}\index{plan d'activités}.

\item Plan d'activités\\ 
C'est un des composants de la stratégie de conduite du système (classe [C]Strategy[/C], voir la page dédiée). On qualifie ce plan de {\em nominal}, au sens où il s'agit d'une spécification des actions de pilotage à entreprendre. Il est structuré par des {\em opérateurs de composition}\index{opérateur de composition}. 

\item Opérateur de composition\\
Quand un tel opérateur porte sur deux ou plusieurs activités, il induit sur ces activités une contrainte temporelle (telle que la précédence, le recouvrement) ou un choix non déterministe (activités alternatives). Quand un tel opérateur porte sur une seule activité, il apporte une précision sur la façon dont cette activité doit être exécutée (par exemple, de façon répétée ou optionnelle). A chaque opérateur de composition correspond une sous-classe de la classe de base [C]Activity[/C].\\

\item Activité non primitive\\
Ces sous-classes d'activité, plus précisément leurs symboles de classes, sont listées à la page "Types prédéfinis, ..."/"Symboles des classes d'entités"). Une activité peut ainsi être définie comme une relation sur d'autres activités, et par conséquent essentiellement caractérisée par son type (de relation) et ses éléments (les activités liées par la relation, qu'on appelle aussi les {\em activités composantes}\index{activités composantes}). Puisque les activités sont des entités, le type de l'activité {\em pAct} est donc accessible par l'appel [P]pAct->ClassSymbol()[/P], et la liste de ses {\em activités composantes} par l'appel [P]pAct->GetElementTab()[/P].

\item Activité primitive\\
On appelle {\em instruction}\index{instruction}, ou {\em activité primitive},\index{activité primitive} une activité d'un type particulier (sous-classe [C]PrimitiveActivity[/C], en ce qu'elle ne spécifie pas de relation sur d'autres activités : une instruction a un type (PRIMITIVE_ACTIVITY), mais sa liste d'éléments est vide. On verra de manière plus précise qu'une activité primitive spécifie non seulement l'opération (l'action), mais aussi son objet (on dit aussi l'objet opéré) et l'opérateur de l'action sur l'objet opéré. 

\item Activité racine du plan\\
Il existe toujours dans un {\em plan nominal} une activité qui n'est élément d'aucune autre activité, c'est-à-dire qui n'est liée à aucune autre activité par un {\em opérateur de composition}. Se référer à cette {\em activité racine}\index{activité racine}, c'est se réferer au {\em plan nominal} lui-même. Les {\em instructions} sont donc les feuilles d'un arbre qui se développe de manière arborescente à partir de l'{\em activité racine} ({\em le plan nominal}) à travers ses {\em composantes}. 

\item Jeu d'activités\\
Un jeu d'activités est une conjonction d'activités primitives, issue de l'examen de l'état (descripteur [C]Situation[/C]) des activités d'un plan. On ne s'intéresse généralement qu'auxactivités "ouvertes" (voir ci-dessous), pour constituer, en un instant donné, un ou plusieurs jeu d'activités candidates à l'exécution. En fonction de la structure du plan (déterminée par les opérateurs de composition), les activités primitives ouvertes sont tel ou tel sous-ensemble des feuilles de l'arborescence du plan. Ainsi, on ne parlera pas de "jeu d'activités" pour désigner les éléments d'une activité non primitive.
\end{itemize}

\item {\bf Procédures de changement d'état}

\item {\bf Action sur le système contrôlé}

\item {\bf Allocation de ressources}

\end{enumerate}
*/
//@{

/**@name La classe de base des activités\\

Cette classe de base n'est pas destinée à être instanciée directement. Elle est seulement le siège de descripteurs et services ainsi hérités par les sous-classes d'activités non primitives et primitives.

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]Activity[/C]} & {\tiny ACTIVITY} & {\tiny activity} & {\tiny } & {\tiny }
\end{tabular}

  {\bf Descripteurs :}\\

\begin{tabular}{ccccl}
 & {\tiny type} & {\tiny classe, symbole, nom} & {\tiny v. par défaut} & {\tiny }\hline
\hline
{\tiny C} & {\tiny TypeAttribute} & {\tiny [C][/C] } & {\tiny ()} & {\tiny }\hline
{\tiny C} & {\tiny BegDate} & {\tiny [C][/C] } & {\tiny ()} & {\tiny }\hline
{\tiny V} & {\tiny MinBegDate} & {\tiny [C][/C] } & {\tiny ()} & {\tiny }\hline
{\tiny V} & {\tiny MaxBegDate} & {\tiny [C][/C] } & {\tiny ()} & {\tiny }\hline
{\tiny V} & {\tiny EndDate} & {\tiny [C][/C] } & {\tiny ()} & {\tiny }\hline
{\tiny V} & {\tiny MinEndDate} & {\tiny [C][/C] } & {\tiny ()} & {\tiny }\hline
{\tiny V} & {\tiny MaxEndDate} & {\tiny [C][/C] } & {\tiny ()} & {\tiny }\hline
{\tiny V} & {\tiny Situation} & {\tiny [C][/C] } & {\tiny ()} & {\tiny }\hline
{\tiny V} & {\tiny ReadyToRun} & {\tiny [C][/C] } & {\tiny ()} & {\tiny }\hline
\end{tabular}
  
  {\bf Méthodes :}

\begin{itemize}
\item [C]OpeningPredicate[/C] ...
\item [C]ClosingPredicate[/C] ...
\item [C]MinBegStatePredicate[/C] ...
\item [C]MaxBegStatePredicate[/C] ...
\end{itemize}

\begin{tabular}{ccccl}
{\tiny type} & {\tiny classe, symbole, nom} & {\tiny corps par défaut} & {\tiny classes des arguments}\hline
\hline
{\tiny } & {\tiny [C]OpeningPredicate[/C] } & {\tiny} & {\tiny [C][/C]} \hline
{\tiny } & {\tiny [C]ClosingPredicate[/C] } & {\tiny} & {\tiny [C][/C]} \hline
{\tiny } & {\tiny [C]MinBegStatePredicate[/C] } & {\tiny} & {\tiny [C][/C]} \hline
{\tiny } & {\tiny [C]MaxBegStatePredicate[/C] } & {\tiny} & {\tiny [C][/C]}
\end{tabular}

*/
//@{
//----------------------------------------------------------------------------
// ACTIVITY
//----------------------------------------------------------------------------
          ///
class Activity : public Entity
{
 public:
  
  /**@name LIFECYCLE */
  
  //@{
  /**@name  Constructeur de base 
     
  Ce constructeur de base ne doit pas être invoqué par l'utilisateur, car aucune instance directe ne doit être créée pour cette classe (de symbole ACTIVITY). Ce constructeur est en fait automatiquement appelé avant le constructeur propre à chaque sous-classe. L'instance créée est alors ajoutée à la liste des instances (indirectes) de la classe {\bf Activity}.\\
  
  {\bf descripteurs :}\\
  Les descripteurs de type {\bf int} représentent un nombre d'unités de temps simulé, à partir du début de la simulation (voir \Db : Manuel de l'utilisateur). La valeur par défaut est en dernière colonne.\index{date de début (activité)}\index{date de fin (activité)} \\
  
  {\scriptsize 
  \noindent\begin{tabular}{lllll}
  SITUATION & situation & état de l'activité & {\bf C\_StateId} & SLEEPING  \\
  BEG\_DATE & begDate & date de début réel & {\bf int} & -9  \\
  MIN\_BEG\_DATE & minBegDate & date de début au plus tôt & {\bf int} & -9   \\
  MAX\_BEG\_DATE & maxBegDate & date de début au plus tard & {\bf int} & -9    \\
  END\_DATE & endDate & date de fin réelle & {\bf int} & -9   \\
  MIN\_END\_DATE & minEndDate & date de fin au plus tôt & {\bf int} & -9    \\
  MAX\_END\_DATE & maxEndDate & date de fin au plus tard & {\bf int} & -9    \\
  \end{tabular}
  }\\
  
  {\bf méthodes :}\\
  
  {\scriptsize 
  \noindent\begin{tabular}{llll}
  OPENING_PREDICATE & openingPredicate & prédicat d'ouverture & {\bf EntityPredicate}  \\
  CLOSING_PREDICATE & closingPredicate & prédicat de fermeture & {\bf EntityPredicate}  \\
  \end{tabular} 
  }
  
  */
  // 
  Activity();  // _no swig_
  
  // à revoir et valider Constructeur par copie profonde. Les constructeurs par copie définis au niveau de chaque sous-classe ne font rien d'autre que d'invoquer le présent constructeur par copie. Les {\em composantes} de l'activité copiée sont elles aussi copiées (à l'aide du constructeur par copie de la sous-classe concernée)
  Activity(const Activity&);  // _no swig_
  
  // Destructeur. Supprime aussi le tableau de tableaux de {\bf PrimitiveActivity} attaché à l'activité en cours de destruction (voir méthode {\bf GetNonConsistentSetsOfInstructions}, page \pageref{defGetNonConsistentSetsOfInstructions}).\label{destructorActivity}
  
  //@}
  
  
  /**@name  ACCESS */
  //@{
  //
  TableauInt*  OpeningWindow();  // _no swig_
  //
  int  GetMinBegDate();  // _no swig_
  //
  int  GetMaxBegDate();  // _no swig_
  //
  int  GetMinEndDate();  // _no swig_
  //
  int  GetMaxEndDate();  // _no swig_
  
  //
  int GetMinBegDateFromUp();
  //
  int GetMaxBegDateFromUp();
  //
  int GetMinEndDateFromUp();
  //
  int GetMaxEndDateFromUp(); 
  
  //
  int GetMinBegDateWithUp();
  //
  int GetMaxBegDateWithUp();
  //
  int GetMinEndDateWithUp();
  //
  int GetMaxEndDateWithUp(); 
  
  // + D5.10
  //
  void AddOperand(Activity*);  // _no swig_    // + JPR 120606
  //
  Activity* RemoveOperand(Activity*);  // _no swig_    // + JPR 120606
  //
  int GetNumberOfOperands();  // _no swig_    // + JPR 120606
  //
  Activity* GetOperand(int);  // _no swig_    // + JPR 120606
  //
  Activity* GetOperand();  // _no swig_    // + JPR 120606
  //
  Activity* GetLastOperand();  // _no swig_    // + JPR 120606
  //
  int GetOperandPosition(Activity*);  // _no swig_    // + JPR 120606

  // virtual int  GetMinBegDateFromDown();  // _no swig_
  // voir note JPR 050901 dans C_Activity.cc
  //
  int  GetMinBegDateFromDown();  // _no swig_
  //
  int  GetMaxBegDateFromDown();  // _no swig_
  //
  int  GetMinEndDateFromDown();  // _no swig_
  //
  int  GetMaxEndDateFromDown();  // _no swig_ 
  
  //
  int  GetMinBegDateWithDown();  // _no swig_
  //
  int  GetMaxBegDateWithDown();  // _no swig_
  //
  int  GetMinEndDateWithDown();  // _no swig_
  //
  int  GetMaxEndDateWithDown();  // _no swig_
  
  /// Vérifie la cohérence des fenêtres d'ouverture et de fermeture, en termes de dates (unités de temps simulé depuis le début de la simulation). Les dates considérées sont celles spécifiées (ou non) au niveau même de l'activité\index{méthode!CheckLocalDates}
  void             CheckLocalDates();
  
  /// Opère la même fonction que {\bf CheckLocaldates}, mais les dates considérées sont celles calculées à partir de celles spécifiées (ou non) au niveau même de l'activité, d'une part, et des ascendants et descendants de l'activités, d'autre part.\index{méthode!CheckDates}
  void             CheckDates();
  
  /// Imprime les fenêtres d'ouverture et de fermeture, en termes de dates (unités de temps simulé depuis le début de la simulation). Les valeurs affichées sont celles attribuées à l'activité elle-même. \index{méthode!PrintLocalDates}
  void             PrintLocalDates();
  
  /// Imprime les fenêtres d'ouverture et de fermeture, en termes de dates (unités de temps simulé depuis le début de la simulation). Les valeurs affichées sont celles attribuées à l'activité elle-même où déduites des dates attribuées aus éléments et/ou aux sur-ensembles.\index{méthode!PrintDates}
  void             PrintDates();
  
  // \ index{méthode!:SetInDepthDatesToDefault}
  void             SetInDepthDatesToDefault();

  // \ index{méthode!:SetInDepthIsOpeningPredicateCheckedNotFalseToDefault}
  void             SetInDepthIsOpeningPredicateCheckedNotFalseToDefault();
  
  //
  void SetInDepthSituationsToSleeping();

  /// Etablit les conditions d'ouverture et de fermeture, en termes de prédicats. Une donnée manquante doit être codée par le pointeur NULL.\index{méthode!SetOpenClosePredicates}
  void             SetOpenClosePredicates(bool (*openPred) (EntityMethod*),                                              bool (*closePred) (EntityMethod*));  // _no swig_ 
  
  // Renvoie {\em vrai} si on a donné un corps au prédicat d'ouverture et {\em faux} sinon.\index{méthode!HasOpeningPredicate}
  //inutile : utiliser Entity::HasBody(OPENING_PREDICATE)
  //bool           HasOpeningPredicate();  // _no swig_
  
  /// Renvoie {\em vrai} si on a donné un corps au prédicat d'ouverture et si ce prédicat renvoie {\em vrai}, et {\em faux} sinon.\index{méthode!TrueOpeningPredicate}
  bool             TrueOpeningPredicate();  // _no swig_
  
  /// Renvoie {\em vrai} si on n'a pas donné un corps au prédicat d'ouverture ou si ce prédicat renvoie {\em vrai}, et {\em faux} sinon.\index{méthode!NotFalseOpeningPredicate}
  bool             NotFalseOpeningPredicate();  // _no swig_
  
  // Renvoie {\em vrai} si on a donné un corps au prédicat de fermeture et {\em faux} sinon.\index{méthode!HasClosingPredicate}
  //inutile : utiliser Entity::HasBody(CLOSING_PREDICATE)
  //bool             HasClosingPredicate();  // _no swig_
  
  /// Renvoie {\em vrai} si on a donné un corps au prédicat de fermeture et si ce prédicat renvoie {\em vrai}, et {\em faux} sinon.\index{méthode!TrueClosingPredicate}
  bool             TrueClosingPredicate();  // _no swig_
  
  /// Renvoie {\em vrai} si on n'a pas donné un corps au prédicat de fermeture ou si ce prédicat renvoie {\em vrai}, et {\em faux} sinon.\index{méthode!NotFalseClosingPredicate}
  bool             NotFalseClosingPredicate();  // _no swig_
  
  /// Renvoie {\em vrai} si la condition d'ouverture est spécifiée et (1) si la date courante est antérieure à la fenêtre de début de l'activité lorsque celle-ci est spécifiée ou (2) si on est dans la fenêtre de début mais le prédicat d'ouverture n'est pas satisfait s'il existe.\index{méthodes!IsOpenConditionNotYetSpécifiée}
  bool        IsOpenConditionNotYetSatisfied();  // _no swig_
  
  /// Renvoie {\em vrai} si la condition de fermeture est spécifiée et (1) si la date courante est antérieure à la fenêtre de fin de l'activité lorsque celle-ci est spécifiée ou (2) si on est dans la fenêtre de fin mais que le prédicat de fermeture n'est pas satisfait s'il existe.\index{méthodes!IsCloseConditionNotYetSpécifiée}
  bool        IsCloseConditionNotYetSatisfied();  // _no swig_
  
  /// Renvoie {\em vrai} si la condition d'ouverture est spécifiée et si la date courante est dans la fenêtre de début de l'activité, lorsque celle-ci est spécifiée, et si le prédicat d'ouverture renvoie {\em vrai} s'il est spécifié.\index{méthodes!IsOpenConditionSatisfied}
  bool        IsOpenConditionSatisfied();  // _no swig_
  
  /// Renvoie {\em vrai} si la condition de fermeture est spécifiée et si la date courante est dans la fenêtre de fin de l'activité, lorsque celle-ci est spécifiée, et si le prédicat de fermeture renvoie {\em vrai} s'il est spécifié.\index{méthodes!IsCloseConditionSatisfied}
  bool        IsCloseConditionSatisfied();  // _no swig_
  
  /// Renvoie {\em vrai} si la condition d'ouverture est spécifiée et si la date courante est postérieure à la fenêtre de début de l'activité, lorsque celle-ci est spécifiée.\index{méthodes!IsOpenConditionNoLongerSatisfiable}
  bool        IsOpenConditionNoLongerSatisfiable();  // _no swig_
  
  /// Renvoie {\em vrai} si la condition d'ouverture est spécifiée et si la date courante est postérieure à la fenêtre de fin de l'activité, lorsque celle-ci est spécifiée.\index{méthodes!IsCloseConditionNoLongerSatisfiable}
  bool        IsCloseConditionNoLongerSatisfiable();  // _no swig_
  
  /// Renvoie {\em vrai} si l'activité est ouverte et si, selon le type de l'activité, une opération ou toutes les premières opérations en jeu à une profondeur quelconque et déjà allouée n'a ou n'ont pas encore eu un début d'exécution lorsque la fin de la fenêtre d'ouverture est atteinte. Renvoie {\em faux} dans les autres cas.\index{méthodes!OperationNotBegunAtMaxBegDate}
  bool        HasOperationNotBegunAtMaxBegDate();  // _no swig_
  bool        HasOperationNotBegun();  // _no swig_
  
  /// Renvoie {\em vrai} si une activité optionelle, ou si une activité optionelle contrainte telle que le descripteur N a une valeur inférieure au descripteur N1 (voir page \pageref{}) contient la présente activité comme élément à une profondeur quelconque, et {\em faux} sinon.\index{méthodes!IsOptional}
  bool        IsOptional() const;  // _no swig_
  
  /// Renvoie {\em vrai} si une activité option contient la présente activité comme élément immediat, et {\em faux} sinon.\index{méthodes!IsImmediateOptional}
  bool        IsImmediateOptional() const;  // _no swig_
  
  /// Renvoie un tableau contenant les activités de type ACTIVITY\_OPTIONAL qui contiennent la présente activité comme élément à une profondeur quelconque. Le tableau comprend aussi les optionnelles contraintes telles que le descripteur N a une valeur inférieure au descripteur N1 (voir page \pageref{})\index{méthodes!GetOptionalContainers}
  pActivityTab*    GetOptionalContainers() const;  // _no swig_
  
  /// Renvoie l'activité de type ACTIVITY\_BEFORE\_ANY ou ACTIVITY\_MEETING\_ANY qui contient la présente activité comme élément à une profondeur quelconque, et NULL si ce n'est pas le cas.\index{méthodes!IsInBeforeOrMeetingAny}
Activity* IsInBeforeOrMeetingAny() const;

  // Renvoie {\em vrai} si une activité itérative contient la présente activité comme élément à une profondeur quelconque, et {\em faux} sinon.\index{méthodes!IsIterated}
  /// Renvoie l'activité itérative qui contient la présente activité comme élément à une profondeur quelconque, et NULL si ce n'est pas le cas.\index{méthodes!IsIterated}
  ActivityIteration*        IsIterated() const;  // _no swig_
  
  /// Si cette activité est composante (unique) d'une activité itérative (forcément unique, par définition), la présente méthode renvoie cette activité itérative, et {\em NULL} sinon.\index{méthodes!IsImmediateIterated}
  ActivityIteration*   IsImmediateIterated() const;  // _no swig_
  
  /// Renvoie un tableau contenant les activités de type ACTIVITY\_ITERATION qui contiennent la présente activité comme élément à une profondeur quelconque. Ce tableau ne contient normalement qu'un seul élément.\index{méthodes!GetIterationContainers}
  pActivityTab*    GetIterationContainers();  // _no swig_
  
  /// Renvoie {\em vrai} si la situation de l'activité est OPEN ou IN_COURSE, et {\em faux} sinon.\index{méthodes!IsOpen}
  bool         IsOpen();
  /// Renvoie {\em vrai} si la situation de l'activité est OPEN, et {\em faux} sinon.\index{méthodes!IsJustOpen}
  bool         IsJustOpen(); // + 091008

  /// Teste si l'activité est encore ouverte en fin de simulation, et émet un avertissement le cas échéant. Généralement invoquée sur l'activité racine d'un plan.\index{méthodes!CheckStillOpenAtSimulationEnd}
  void         CheckStillOpenAtSimulationEnd();
  
  /// Renvoie un sous-ensemble des opérations mise en jeu dans l'activité (à une profondeur quelconque) : seulement celles à la fois en cours d'exécution et suivies par aucune autre dans cette activité.\index{méthodes!LastExecutingOperations}
  pEntityTab*  LastExecutingOperations();
  /// Renvoie {\em vrai} si les opérations renvoyées par la méthode ci-dessus sont toutes permanentes.\index{méthodes!AreLastExecutingOperationsPermanent}
  bool         AreLastExecutingOperationsPermanent();
  /// Renvoie un sous-ensemble des opérations mise en jeu dans l'activité (à une profondeur quelconque) : seulement celles suivies par aucune autre dans cette activité.\index{méthodes!LastOperations}
  pEntityTab*  LastOperations();
  /// Renvoie {\em vrai} si les opérations renvoyées par la méthode ci-dessus sont toutes permanentes.\index{méthodes!AreLastOperationsPermanent}
  bool         AreLastOperationsPermanent();
  /// Renvoie {\em vrai} si une des opérations renvoyées par la méthode ci-dessus est permanente.\index{méthodes!HasLastOperationPermanent}
  bool         HasLastOperationPermanent();                // + JPR 110314 (5.7)
  /// Renvoie la date la plus tardive de mise à jour de l'opération parmi les dernières composantes possibles dans cette activité.\index{méthodes!LastActivityUpdatingDate}
  int          LastActivityUpdatingDate();
  /// Renvoie un sous-ensemble des activités primitives mise en jeu dans l'activité (à une profondeur quelconque) : seulement celles suivies par aucune autre dans cette activité.\index{méthodes!LastPrimitives}
  pEntityTab*  LastPrimitives();   // !! not used !!
  /// Renvoie un sous-ensemble des activités primitives mise en jeu dans l'activité (à une profondeur quelconque) : seulement celles précédées par aucune autre dans cette activité.\index{méthodes!FirstPrimitives}
  pEntityTab*  FirstPrimitives();   // !! not used !!
  /// Renvoie un sous-ensemble des activités primitives mise en jeu dans l'activité (à une profondeur quelconque) : seulement celles dont la situation est IN\_COURSE.\index{méthodes!InCoursePrimitiveList}
  pEntityTab*  InCoursePrimitiveList();
  /// Renvoie un sous-ensemble des activités primitives mise en jeu dans l'activité (à une profondeur quelconque) : seulement celles dont la situation est OPEN ou IN\_COURSE.\index{méthodes!OpenPrimitiveList}
  pEntityTab*  OpenPrimitiveList();
  /// Renvoie un sous-ensemble des activités mise en jeu dans l'activité (à une profondeur quelconque) : seulement celles dont la situation est OPEN ou IN\_COURSE.\index{méthodes!OpenActivityList}
  pEntityTab*  OpenActivityList();
  /// Renvoie la liste des activités primitives mise en jeu dans l'activité.\index{méthodes!PrimitiveList}
  pEntityTab*  PrimitiveList();
  /// Affecte l'entité passée en argument comme unique objet opéré de toutes les activités primitives mise en jeu dans l'activité.\index{méthodes!PrimitiveList}
  void         AssignOperatedObjectToEachPrimitive(BasicEntity*);
  
  
  /// Renvoie une liste de couples d'activités. Le premier élément d'un couple est l'activité qui suit {\em this} dans une séquence ({\bf ActivityBefore} ou {\bf ActivityMeeting}). Le second est l'activité séquence elle-même. Aucun couple n'est placé dans la liste renvoyée si {\em this} n'est pas incluse dans une séquence. Si {\em this} est incluse dans une séquence en dernière position, le couple correspondant n'est pas placé dans la liste.\index{méthodes!NextActivityInSequences}
  pEntityDTab*  NextActivityInSequences();
  // 
  pEntityDTab*  NextActivityInSequence(Activity*);
  
  /// Renvoie {\em vrai} si l'activité passée en argument est ou peut être la dernière composante de {\em this}, et {\em faux} sinon.\index{méthodes!IsOrMayBeUniqueLast}
  bool         IsOrMayBeUniqueLast(Activity*);
  
  //
  bool         AllPrimitivesAlreadyUpdatedAtOrBefore(int clock);
  //
  IntInterval* AllPrimitivesUpdatedAtSameClock();
  //@}  
  
  
  /**@name OPERATIONS */
  //@{
  
  /**@name Opérations de changement d'état \label{defMethChangeState}
   */
  //@{
  
  /**@name Changements impératifs 
     
  \label{changeImper}La méthode {\bf MakeReadyToRun} doit en principe être invoquée dans le programme de l'utilisateur. Ce n'est pas le cas des autres méthodes, qui ne sont normalement invoquées que dans les procédures prédéfinies de \CD. Elle figurent ici pour compléter la description du fonctionnement de ces procédures. 
  */
  //@{
  /// Opère le changement d'état de SLEEPING à WAITING. Cet appel doit être effectué dès qu'on souhaite que l'activité puisse être examinée pour produire éventuellement des effets. De façon standard, ce message est adressé à l'{\em activité racine} d'un plan au début de la période de simulation, après la création en mémoire de l'activité. \index{méthodes!MakeReadyToRun}\label{defReadyToRun}
  void MakeReadyToRun();  // _no swig_
  ///   
  void UpdateStatus();  // _no swig_ 
  ///   
  void ResetDisjunctionInDepth();  // _no swig_  
  ///   
  void DeleteDisjunctionInDepth();  // _no swig_ 
  ///
  void PropagateSituationChangeToMothers();  // _no swig_
  //
  void ResetMaxBegDateInDepth();  // _no swig_
 
  /// 
  pEntityTab*   SearchRoots();
  /// 
  Activity*     HasUniqueRoot(int updown, Activity* source, Activity* root);  // _no swig_  
  
  /// 
  Activity*   RootActivity();
  /// 
  void CleanLocalValidationAttributes();  // _no swig_    
  /// 
  void CleanValidationFromChain();  // _no swig_   
  /// 
  void RemoveFromValidationFromChain(Activity*);  // _no swig_   // + JPR 110505

  /// Opère le changement d'état à la valeur SLEEPING pour l'activité qui reçoit le message et pour ses activités composantes, s'il elle en a.\index{méthodes!TurnToSleeping}
  void TurnToSleeping();  // _no swig_
  /// \index{méthodes!TurnToOpen}  
  void TurnToOpen(int updown, Activity* source);  // _no swig_
  ///\index{méthodes!TurnToWaiting}   
  void TurnToWaiting(int updown, Activity* source);  // _no swig_
  ///\index{méthodes!TurnToClosed}   
  void TurnToClosed(int updown, Activity* source);  // _no swig_
  ///\index{méthodes!TurnToNoLongerOpenable}   
  void TurnToNoLongerOpenable(int updown, Activity* source);  // _no swig_ 
  /// Affecte à l'activité l'état IN\_COURSE, si elle ne l'a pas déjà et si elle n'est pas CANCELLED, puis propage cette valeur aux activités dont elle est {\em composante}. Lève l'exception {\bf PreconditionOnState} si l'état n'est pas OPEN. Si l'activité qui reçoit ce message est {\em composante} d'une activité de type ACTIVITY\_DISJUNCTION, toutes les autres {\em activités alternatives} de la disjonction sont annulées (passage à l'état CANCELLED). Si l'activité qui reçoit ce message est {\em composante} d'une activité de type ACTIVITY\_SET, toutes les autres {\em activités} de la séquence sont mises en sommeil (passage à l'état SLEEPING).\index{méthodes!TurnToInCourse} 
  void     TurnToInCourse();                        // _no swig_
  /// Exclusivement utilisé lors du traitement d'une {\bf ActivityCoending} ou d'une {\bf ActivityEquality}. Lorsqu'une composante devient CLOSED, on s'apprête à stopper toutes les opérations qui ne sont pas terminées (la situation devient TO_STOP). Si le degré de progression a été mis à jour à la date courante, on ferme la primitive, et la situation de l'opération devient TERMINATED. \index{méthodes!TurnExecutingOperationToStop} 
  void     TurnExecutingOperationToStop();                        // _no swig_
  
  //@}  // fin changement d'état impératifs
  
  //@}  // fin Opérations de changement d'état
  
  /**@name Propagation de changement d'état 
     
  Ces méthodes ne sont pas définies au niveau de la classe générale des activités, mais au niveau de ses sous-classes, puisqu'elles en traduisent pour partie le sens. Par exemple, une relation de type MEETING entre deux activités {\em a1} et {\em a2} implique que {\em a2} soit ouverte dès que {\em a1} est fermée. Ceci est codé dans la méthode {\bf UpdateIfSonClosed} de la classe {\bf ActivityMeeting}, déclenchée quand l'état de {\em a1} passe à CLOSED.
  */
  //@{
  ///
  void PropagateWaitingToSons(Activity*);  // _no swig_
  /// 
  void PropagateOpenToSons(Activity*);  // _no swig_
  /// 
  void PropagateClosedToSons(Activity*);  // _no swig_
  /// 
  void PropagateNoLongerOpenableToSons(Activity*);  // _no swig_
  
  /// 
  void UpdateIfSonWaiting(Activity*);  // _no swig_
  /// 
  void UpdateIfSonOpen(Activity*);  // _no swig_
  /// 
  void UpdateIfSonClosed(Activity*);  // _no swig_
  /// 
  void UpdateIfSonNoLongerOpenable(Activity*);  // _no swig_
  
  //@}  // fin Propagation de changement d'état  
  
  /**@name Tests d'exécutabilité */
  //@{
  /// 
  bool          ValidateWaiting(C_UpDown, Activity*);  // _no swig_
  /// 
  bool          ValidateOpen(C_UpDown, Activity*);  // _no swig_
  /// 
  bool          ValidateClosed(C_UpDown, Activity*);  // _no swig_
  
  /// 
  bool          CheckWaitingGeneralPrecondition();  // _no swig_
  ///
  bool          CheckOpenGeneralPrecondition(C_UpDown, Activity*);  // _no swig_
  /// 
  bool          CheckClosedGeneralPrecondition();  // _no swig_
  /// 
  bool          IsClosedAuthorizedByOperation();  // _no swig_
  
  /// 
  bool CheckSonsIfWaiting();  // _no swig_
  /// 
  bool CheckSonsIfWaiting(Activity*);  // _no swig_
  /// 
  bool CheckSonsIfOpen();  // _no swig_
  /// 
  bool CheckSonsIfOpen(Activity*);  // _no swig_
  /// 
  bool CheckSonsIfClosed();  // _no swig_
  /// 
  bool CheckSonsIfClosed(Activity*);  // _no swig_
  
  /// 
  bool CheckIfSonWaiting(Activity*);  // _no swig_
  /// 
  bool CheckIfSonOpen(Activity*);  // _no swig_
  /// 
  bool CheckIfSonClosed(Activity*);  // _no swig_
  
  //@}  // fin Tests d'exécutabilité 
  

        /// Affiche la situation de l'activité et de ses éléments. k est l'indentation initiale
  void  DisplaySituation(int k);  // _no swig_
        /// A le même effet que DisplaySituation(0).
  void  DisplaySituation();  // _no swig_
  
        /// Affiche la situation et les fenêtres d'ouverture/fermeture de l'activité et de ses éléments. k est l'indentation initiale
  void  DisplayOpenCloseConditions(int k);  // _no swig_
        /// A le même effet que DisplayOpenCloseConditions(0).
  void  DisplayOpenCloseConditions();  // _no swig_
  
        
  
  
  //@}  // fin opérations
  
 protected:
  
 private:
};  // _end class_
  
//@}  // fin classe de base

//----------------------------------------------------------------------------

#endif  // _C_ACTIVITY_HH

 
