//============================================================================
// FILE: C_CoreConstantDescriptor.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête des divers descripteurs constants de Control-Diese
// 
// AUTHORS: Baali/Rellier (INRA/UBIA, Toulouse)
// REFERENCE : "Fondements ontologiques des systèmes pilotés"
//             R. Martin-Clouaire, J.-P. Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _C_CORECONSTDESCRIPTOR_HH
#define _C_CORECONSTDESCRIPTOR_HH

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//
#include "C_CoreTypes.h"

/**@name Les descripteurs prédéfinis */
//@{
/**@name Les descripteurs constants

\begin{tabular}{lllll}
type&nom C++&symbole de classe&nom de classe&sens de la valeur \\
&OperatedPackAttribute&OPERATED_PACK_ATTRIBUTE&operatedPackAttribute& \\
&InstructionSetDisjunctionAttribute&INSTRUCTION_SET_DISJUNCTION_ATTRIBUTE&instructionSetDisjunctionAttribute& \\
&InvolvedIntQuantity&INVOLVED_INT_QUANTITY&involvedIntQuantity& \\
&InvolvedFloatQuantity&INVOLVED_FLOAT_QUANTITY&involvedFloatQuantity& \\
&NoOperationSpec&NO_OPERATION_SPEC&noOperationSpec& \\
&DistributionMode&DISTRIBUTION_MODE&distributionMode& \\
&HasSituationChanged&HAS_SITUATION_CHANGED&hasSituationChanged \\
&IsOneShot&IS_ONE_SHOT&isOneShot \\
&ObjectSetType&OBJECT_SET_TYPE&objectSetType& \\
&PrimitiveActivityAttribute&PRIMITIVE_ACTIVITY_ATTRIBUTE&primitiveActivityAttribute& \\
&AllocationAttribute&ALLOCATION_ATTRIBUTE&allocationAttribute& \\
&SetAllocationAttribute&SET_ALLOCATION_ATTRIBUTE&setAllocationAttribute& \\
&SetAllocationsAttribute&SET_ALLOCATIONS_ATTRIBUTE&setAllocationsAttribute& \\
&CriterionAttribute&CRITERION_ATTRIBUTE&criterionAttribute& \\
&ValueForSelection&VALUE_FOR_SELECTION&valueForSelection& \\
&GlobalTargetValue&GLOBAL_TARGET_VALUE&globalTargetValue& \\
&GlobalAntiTargetValue&GLOBAL_ANTI_TARGET_VALUE&globalAntiTargetValue& \\
&ArbAttribute&ARB_ATTRIBUTE&arbAttribute& \\
&OperatedEntityId&OPERATED_ENTITY_ID&operatedEntityId& \\
CurrentOperatedObject&CURRENT_OPERATED_OBJECT&currentOperatedObject& \\
&Step&STEP&step& \\
&ProgressionMode&PROGRESSION_MODE&progressionMode& \\
&SuspensionThreshold&SUSPENSION_THRESHOLD&suspensionThreshold& \\
&UnitSpeed&UNIT_SPEED&unitSpeed& \\
&QuantityDescriptorId&QUANTITY_DESCRIPTOR_ID&quantityDescriptorId& \\
&IntQuantitySpeed&INT_QUANTITY_SPEED&intQuantitySpeed& \\
&FloatQuantitySpeed&FLOAT_QUANTITY_SPEED&floatQuantitySpeed& \\
&TimeSpeed&TIME_SPEED&timeSpeed& \\
&ResourceUsage&RESOURCE_USAGE&resourceUsage& \\
&OperationSpecContainer&OPERATION_SPEC_CONTAINER&operationSpecContainer& \\
&AchievementDegreeIncrement&ACHIEVEMENT_DEGREE_INCREMENT&achievementDegreeIncrement& \\
&IsSingle&IS_SINGLE&isSingle& \\
&IsDiscreteState&IS_DISCRETE_STATE&isDiscreteState& \\
&IsPermanent&IS_PERMANENT&isPermanent& \\
&IsReusable&IS_REUSABLE&isReusable& \\
&ResourceEntity&RESOURCE_ENTITY&resourceEntity& \\
&ResourceEntityMemory&RESOURCE_ENTITY_MEMORY&resourceEntityMemory& \\
&ResourceCapacitySupply&RESOURCE_CAPACITY_SUPPLY&resourceCapacitySupply& \\
&ResourceCapacityDemand&RESOURCE_CAPACITY_DEMAND&resourceCapacityDemand& \\
&DescribedResource&DESCRIBED_RESOURCE&describedResource& \\
&IsDestination&IS_DESTINATION&isDestination& \\
&IntLowerLevel&INT_LOWER_LEVEL&intLowerLevel& \\
&IntUpperLevel&INT_UPPER_LEVEL&intUpperLevel& \\
&FloatLowerLevel&FLOAT_LOWER_LEVEL&floatLowerLevel& \\
&FloatUpperLevel&FLOAT_UPPER_LEVEL&floatUpperLevel& \\
&TimeAvailabilityAttribute&TIME_AVAILABILITY_ATTRIBUTE&timeAvailabilityAttribute& \\
&StateAvailabilityAttribute&STATE_AVAILABILITY_ATTRIBUTE&stateAvailabilityAttribute& \\
&CapacityAvailabilityAttribute&CAPACITY_AVAILABILITY_ATTRIBUTE&capacityAvailabilityAttribute& \\
&ResourceList&RESOURCE_LIST&resourceList& \\
&ResourceUsed&RESOURCE_USED&resourceUsed& \\
&OperationUsed&OPERATION_USED&operationUse& \\
&AvailabilityMode&AVAILABILITY_MODE&availabilityMode& \\
&AvailabilityTimetableAttribute&AVAILABILITY_TIMETABLE_ATTRIBUTE&availabilityTimetableAttribute& \\
&StateDomain&STATE_DOMAIN&stateDomain& \\
&OccurrenceCrossDomainAttribute&OCCURRENCE_CROSS_DOMAIN_ATTRIBUTE&occurrenceCrossDomainAttribute& \\
&StateDomain&STATE_DOMAIN&stateDomain& \\
&BinaryOperatorId&BINARY_OPERATOR_ID&binaryOperatorId& \\
&AcceptabilityThreshold&ACCEPTABILITY_THRESHOLDacceptabilityThreshold&& \\
&ObjectEntitySpecAttribute&OBJECT_ENTITY_SPEC_ATTRIBUTE&objectEntitySpecAttribute& \\
&PerformerEntitySpecAttribute&PERFORMER_ENTITY_SPEC_ATTRIBUTE&performerEntitySpecAttribute& \\
&ResourceEntitySpecAttribute&RESOURCE_ENTITY_SPEC_ATTRIBUTE&resourceEntitySpecAttribute& \\
&ContinuationMode&CONTINUATION_MODE&continuationMode& \\
&AllocationPriorityDegree&ALLOCATION_PRIORITY_DEGREE&allocationPriorityDegree& \\
&PriorityDegree&PRIORITY_DEGREE&priorityDegree& \\
&ProportionalityMode&PROPORTIONALITY_MODE&proportionalityMode& \\
&SelectorFctId&SELECTOR_FCT_ID&selectorFctId& \\
&SelectorFctArg&SELECTOR_FCT_ARG&selectorFctArg& \\
&DevelopmentFailure&DEVELOPMENT_FAILURE&developmentFailure& \\
&TransferQuantityId&TRANSFER_QUANTITY_ID&transferQuantityId& \\
&TransferQuantityOption&TRANSFER_QUANTITY_OPTION&transferQuantityOption& \\
&TransferQuantityIntValue&TRANSFER_QUANTITY_INT_VALUE&transferQuantityIntValue& \\
&TransferRemQuantityIntValue&TRANSFER_REM_QUANTITY_INT_VALUE&transferRemQuantityIntValue& \\
&TransferQuantityFloatValue&TRANSFER_QUANTITY_FLOAT_VALUE&transferQuantityFloatValue& \\
&TransferRemQuantityFloatValue&TRANSFER_REM_QUANTITY_FLOAT_VALUE&transferRemQuantityFloatValue& \\
&Controlled&CONTROLLED&controlled& \\
&ResourceConstraintList&RESOURCE_CONSTRAINT_LIST&resourceConstraintList& \\
&AttachedEntity&ATTACHED_ENTITY&attachedEntity& \\
&ActivityListAttribute&ACTIVITY_LIST_ATTRIBUTE&activityListAttribute& \\
&IntInfSupport&INT_INF_SUPPORT&intInfSupport& \\
&IntSupSupport&INT_SUP_SUPPORT&intSupSupport& \\
&IntInfCore&INT_INF_CORE&intInfCore& \\
&IntSupCore&INT_SUP_CORE&intSupCore& \\
&FloatInfSupport&FLOAT_INF_SUPPORT&floatInfSupport& \\
&FloatSupSupport&FLOAT_SUP_SUPPORT&floatSupSupport& \\
&FloatInfCore&FLOAT_INF_CORE&floatInfCore& \\
&FloatSupCore&FLOAT_SUP_CORE&floatSupCore& \\
&GoalAttribute& GOAL_ATTRIBUTE&goalAttribute& \\
&CriterionValue&CRITERION_VALUE&criterionValue& \\
&CriterionSetSubNodes&CRITERION_SET_SUB_NODES&criterionSetSubNodes& \\
&TargetWeight&TARGET_WEIGHT&targetWeight& \\
&AntiTargetWeight&ANTI_TARGET_WEIGHT&antiTargetWeight& \\
&ActivitySetDefaultUtility&ACTIVITY_SET_DEFAULT_UTILITY&activitySetDefaultUtility& \\
&ActivitySetSatisfactionThreshold&ACTIVITY_SET_SATISFACTION_THRESHOLD&activitySetSatisfactionThreshold& \\

\end{tabular}
*/
//@{


//=X===============X=============================X============================
// System
//............................................................................

class OperatedPackAttribute : public EntityConstantDescriptor
{
public:
  OperatedPackAttribute();
};

//=X===============X=============================X============================
// Activity/Activities
//............................................................................

class AllocationPriorityDegree : public IntConstantDescriptor
{      // + JPR 100608
public:
  AllocationPriorityDegree();
};

class InstructionSetDisjunctionAttribute : public EntityConstantDescriptor
{
public:
  InstructionSetDisjunctionAttribute();
};

class InvolvedIntQuantity : public IntConstantDescriptor
{ 
public:
  InvolvedIntQuantity();
};

class InvolvedFloatQuantity : public FloatConstantDescriptor
{ 
public:
  InvolvedFloatQuantity();
};

class NoOperationSpec : public IntConstantDescriptor
{
public:
  NoOperationSpec();
};

class DistributionMode : public IntConstantDescriptor
{
public:
  DistributionMode();
};

class HasSituationChanged : public IntConstantDescriptor  // + JPR 061207
{
public:
  HasSituationChanged();
};

class IsOneShot : public IntConstantDescriptor  // + JPR 071031
{
public:
  IsOneShot();
};

class MaxBegStateReached : public IntConstantDescriptor  // + JPR 071031
{
public:
  MaxBegStateReached();
};

class ViolatedConstraintTabAttribute : public EntityTabConstantDescriptor  // + JPR 100504
{
public:
  ViolatedConstraintTabAttribute();
};

//=X===============X=============================X============================
// ActivitySet
//............................................................................

class ObjectSetType : public IntConstantDescriptor
{
public:
  ObjectSetType();
};

class InstructionSetSubNodes : public EntityConstantDescriptor
{
public:
  InstructionSetSubNodes();
};

class PrimitiveActivityAttribute : public EntityConstantDescriptor
{
public:
  PrimitiveActivityAttribute();
};

class AllocationAttribute : public EntityConstantDescriptor
{
public:
  AllocationAttribute();
};

class SetAllocationAttribute : public EntityConstantDescriptor
{
public:
  SetAllocationAttribute();
};

class SetAllocationsAttribute : public EntityTabConstantDescriptor
{
public:
  SetAllocationsAttribute();
};

class CriterionAttribute : public EntityConstantDescriptor
{
public:
  CriterionAttribute();
};

class ValueForSelection : public FloatConstantDescriptor
{
public:
  ValueForSelection();
};

class GlobalTargetValue : public FloatConstantDescriptor
{
public:
  GlobalTargetValue();
};

class GlobalAntiTargetValue : public FloatConstantDescriptor
{
public:
  GlobalAntiTargetValue();
};

class ResourceExclusionListAttribute : public EntityTabConstantDescriptor // + JPR 1005005
{
public:
  ResourceExclusionListAttribute();
};

class ArbAttribute : public EntityConstantDescriptor
{
public:
  ArbAttribute();
};

//=X===============X=============================X============================
// Operation
//............................................................................

class PriorityDegree : public IntConstantDescriptor
{
public:
  PriorityDegree();
};

class OperatedEntityId : public IntConstantDescriptor
{
public:
  OperatedEntityId();
};

class CurrentOperatedObject : public EntityConstantDescriptor
{
public:
  CurrentOperatedObject();
};

class Step : public IntConstantDescriptor
{
public:
  Step();
};

class ProgressionMode : public IntConstantDescriptor
{
public:
  ProgressionMode();
};

class SuspensionThreshold : public FloatConstantDescriptor
{
public:
  SuspensionThreshold();
};

//............................................................................

class UnitSpeed : public IntConstantDescriptor
{
public:
  UnitSpeed();
};

class QuantityDescriptorId : public IntConstantDescriptor
{
public:
  QuantityDescriptorId();
};

class IntQuantitySpeed : public IntConstantDescriptor
{
public:
  IntQuantitySpeed();
};

class FloatQuantitySpeed : public FloatConstantDescriptor
{
public:
  FloatQuantitySpeed();
};

class TimeSpeed : public FloatConstantDescriptor
{
public:
  TimeSpeed();
};

class ResourceUsage : public EntityConstantDescriptor
{ 
public:
  ResourceUsage();
};

class OperationSpecContainer : public EntityConstantDescriptor
{ 
public:
  OperationSpecContainer();
};

class AchievementDegreeIncrement : public FloatConstantDescriptor
{ 
public:
  AchievementDegreeIncrement();
};


//=X===============X=============================X============================
// Resource
//............................................................................

class IsPermanent : public IntConstantDescriptor  // + JPR 110225
{
 public:
  IsPermanent();
};

class IsSingle : public IntConstantDescriptor
{
 public:
  IsSingle();
};

class IsDiscreteState : public IntConstantDescriptor
{
 public:
  IsDiscreteState();
};

class IsReusable : public IntConstantDescriptor
{
 public:
  IsReusable();
};

class ResourceEntity : public EntityConstantDescriptor
{
 public:
  ResourceEntity();
};

class ResourceEntityMemory : public EntityConstantDescriptor
{
 public:
  ResourceEntityMemory();
};

class ResourceCapacitySupply : public EntityConstantDescriptor
{
 public:
  ResourceCapacitySupply();
};

class ResourceCapacityDemand : public EntityConstantDescriptor
{
 public:
  ResourceCapacityDemand();
};

class DescribedResource : public EntityConstantDescriptor
{
 public:
  DescribedResource();
};

class IsDestination : public IntConstantDescriptor
{
 public:
  IsDestination();
};

class IntLowerLevel : public IntConstantDescriptor
{
public:
  IntLowerLevel();
};

class IntUpperLevel : public IntConstantDescriptor
{
public:
  IntUpperLevel();
};

class FloatLowerLevel : public FloatConstantDescriptor
{
public:
  FloatLowerLevel();
};

class FloatUpperLevel : public FloatConstantDescriptor
{
public:
  FloatUpperLevel();
};

class TimeAvailabilityAttribute : public EntityConstantDescriptor
{
public:
  TimeAvailabilityAttribute();
};

class StateAvailabilityAttribute : public EntityTabConstantDescriptor
{
public:
  StateAvailabilityAttribute();
};

class CapacityAvailabilityAttribute : public EntityTabConstantDescriptor
{
public:
  CapacityAvailabilityAttribute();
};


//=X===============X=============================X============================
// ResourceCapacity
//............................................................................

class NumberOfTransferedElements : public IntConstantDescriptor
{
public:
  NumberOfTransferedElements();
};


//=X===============X=============================X============================
// ResourceAllocation
//............................................................................

class ResourceList : public EntityTabConstantDescriptor
{
 public:
  ResourceList();
};

class ResourceUsed : public EntityConstantDescriptor
{
 public:
  ResourceUsed();
};

class OperationUsed : public EntityConstantDescriptor
{
 public:
  OperationUsed();
};

//=X===============X=============================X============================
// Constraints
//............................................................................

class AvailabilityMode : public IntConstantDescriptor
{
public:
  AvailabilityMode();
};

class AvailabilityTimetableAttribute : public EntityConstantDescriptor
{
public:
  AvailabilityTimetableAttribute();
};

class StateDomain : public DescValueSpecTabConstantDescriptor
{
public:
  StateDomain();
};

class OccurrenceCrossDomainAttribute : public EntityConstantDescriptor
{
public:
  OccurrenceCrossDomainAttribute();
};

class BinaryOperatorId : public IntConstantDescriptor
{
public:
  BinaryOperatorId();
};

class AcceptabilityThreshold : public IntConstantDescriptor
{
public:
  AcceptabilityThreshold();
};

//=X===============X=============================X============================
// Spec
//............................................................................

class ObjectEntitySpecAttribute : public EntitySpecConstantDescriptor
{
 public:
  ObjectEntitySpecAttribute();
};

/* obsolete 060207
class OperationEntitySpecAttribute : public EntitySpecConstantDescriptor
{
 public:
  OperationEntitySpecAttribute();
};
*/

class PerformerEntitySpecAttribute : public EntitySpecConstantDescriptor
{
 public:
  PerformerEntitySpecAttribute();
};

class ResourceEntitySpecAttribute : public EntitySpecConstantDescriptor
{
 public:
  ResourceEntitySpecAttribute();
};

class ContinuationMode : public IntConstantDescriptor
{
public:
  ContinuationMode();
};

class ProportionalityMode : public IntConstantDescriptor
{
public:
  ProportionalityMode();
};

class SelectorFctId : public IntConstantDescriptor
{
public:
  SelectorFctId();
};

class SelectorFctArg : public IntConstantDescriptor
{
public:
  SelectorFctArg();
};

class DevelopmentFailure : public IntConstantDescriptor
{
public:
  DevelopmentFailure();
};

class TransferQuantityId : public IntConstantDescriptor
{
public:
  TransferQuantityId();
};

class TransferQuantityOption : public IntConstantDescriptor
{
public:
  TransferQuantityOption();
};

class TransferQuantityIntValue : public IntConstantDescriptor
{
public:
  TransferQuantityIntValue();
};

class TransferRemQuantityIntValue : public IntConstantDescriptor
{
public:
  TransferRemQuantityIntValue();
};

class TransferQuantityFloatValue : public FloatConstantDescriptor
{
public:
  TransferQuantityFloatValue();
};

class TransferRemQuantityFloatValue : public FloatConstantDescriptor
{
public:
  TransferRemQuantityFloatValue();
};



//=X===============X=============================X============================
// Location
//............................................................................




//=X===============X=============================X============================
// System
//............................................................................

class Controlled : public IntConstantDescriptor
{
public:
  Controlled();
};

class AttachedEntity : public EntityConstantDescriptor
{
public:
  AttachedEntity();
};

class ResourceConstraintList : public EntityTabConstantDescriptor
{
public:
  ResourceConstraintList();
};

//=X===============X=============================X============================
// Strategy 
//............................................................................

class TargetWeight : public FloatConstantDescriptor
{ 
public:
  TargetWeight();
};

class AntiTargetWeight : public FloatConstantDescriptor
{ 
public:
  AntiTargetWeight();
};

class ActivitySetDefaultUtility : public FloatConstantDescriptor
{ 
public:
  ActivitySetDefaultUtility();
};

class ActivitySetSatisfactionThreshold : public FloatConstantDescriptor
{ 
public:
  ActivitySetSatisfactionThreshold();
};

//=X===============X=============================X============================
// ActivityPlan
//............................................................................

//=X===============X=============================X============================


//=X===============X=============================X============================
// Goal
//............................................................................

class ActivityListAttribute : public EntityTabConstantDescriptor
{ 
public:
  ActivityListAttribute();
};

class IntInfSupport : public IntConstantDescriptor
{ 
public:
  IntInfSupport();
};

class IntSupSupport : public IntConstantDescriptor
{ 
public:
  IntSupSupport();
};

class IntInfCore : public IntConstantDescriptor
{ 
public:
  IntInfCore();
};

class IntSupCore : public IntConstantDescriptor
{ 
public:
  IntSupCore();
};

class FloatInfSupport : public FloatConstantDescriptor
{ 
public:
  FloatInfSupport();
};

class FloatSupSupport : public FloatConstantDescriptor
{ 
public:
  FloatSupSupport();
};

class FloatInfCore : public FloatConstantDescriptor
{ 
public:
  FloatInfCore();
};

class FloatSupCore : public FloatConstantDescriptor
{ 
public:
  FloatSupCore();
};

//=X===============X=============================X============================
// Criterion
//............................................................................

class GoalAttribute : public EntityConstantDescriptor
{
public:
  GoalAttribute();
};

class CriterionValue : public FloatConstantDescriptor
{ 
public:
  CriterionValue();
};

class CriterionSetSubNodes : public EntityTabConstantDescriptor
{
public:
  CriterionSetSubNodes();
};
//@}

#endif  // _C_CORECONSTDESCRIPTOR_HH
