//============================================================================
// FILE: C_CoreMethod.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête des diverses classes de méthodes de Control-Diese
// 
// AUTHORS: Baali/Rellier (INRA/UBIA, Toulouse)
// REFERENCE : "Fondements ontologiques des systèmes pilotés"
//             R. Martin-Clouaire, J.-P. Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _C_COREMETHOD_HH
#define _C_COREMETHOD_HH

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//
#include "Method.h"

/**@name Les méthodes prédéfinies et leurs arguments
 */
//@{

/**@name Les méthodes prédéfinies
\begin{tabular}{lllll}
type&nom C++&symbole de classe&nom de classe&sens de la valeur\\
&OpeningPredicate&OPENING_PREDICATE&openingPredicate& \\
&ClosingPredicate&CLOSING_PREDICATE&closingPredicate& \\
&MinBegStatePredicate&MIN_BEG_STATE_PREDICATE&minBegStatePredicate& \\
&MaxBegStatePredicate&MAX_BEG_STATE_PREDICATE&maxBegStatePredicate& \\
&ModifyInOpenSituation&MODIFY_IN_OPEN_SITUATION&modifyInOpenSituation& \\
&UpdateReactivatedSon&UPDATE_REACTIVATED_SON&updateReactivatedSon& \\
&UpdateNextWhenSonOpen&UPDATE_NEXT_WHEN_SON_OPEN&updateNextWhenSonOpen& \\
&UpdatePrevWhenSonOpen&UPDATE_PREV_WHEN_SON_OPEN&updatePrevWhenSonOpen& \\
&UpdateNextWhenSonClosed&UPDATE_NEXT_WHEN_SON_CLOSED&updateNextWhenSonClosed& \\
&UpdatePrevWhenSonClosed&UPDATE_PREV_WHEN_SON_CLOSED&updatePrevWhenSonClosed& \\
&StateTransitionProcedure&STATE_TRANSITION_PROCEDURE&stateTransitionProcedure& \\
&FeasibilityCondition&FEASIBILITY_CONDITION&feasibilityCondition& \\
&SetUserUnitSpeed&SET_USER_UNIT_SPEED&setUserUnitSpeed&Calcule la valeur, l'affecte à UNIT_SPEED et la renvoie. \\
&SetUserIntQuantitySpeed&SET_USER_INT_QUANTITY_SPEED&setUserIntQuantitySpeed&Calcule la valeur, l'affecte à INT_QUANTITY_SPEED et la renvoie. \\
&SetUserFloatQuantitySpeed&SET_USER_FLOAT_QUANTITY_SPEED&setUserFloatQuantitySpeed&Calcule la valeur, l'affecte à FLOAT_QUANTITY_SPEED et la renvoie. \\
&SetUserTimeSpeed&SET_USER_TIME_SPEED&setUserTimeSpeed&Calcule la valeur, l'affecte à TIME_SPEED et la renvoie. \\
&InvolvedEntities&INVOLVED_ENTITIES&involvedEntities& \\
&CapacityOfferDemandComparison&CAPACITY_OFFER_DEMAND_COMPARISON&capacityOfferDemandComparison& \\
&StateRelatedPredicate&STATE_RELATED_PREDICATE&stateRelatedPredicate& \\
&HoldingCondition&HOLDING_CONDITION&holdingCondition& \\
&SetThisTimeSlotBegValue&SET_THIS_DOMAIN_BEG_VALUE&setThisTimeSlotBegValue& \\
&SetThisTimeSlotEndValue&SET_THIS_DOMAIN_END_VALUE&setThisTimeSlotEndValue& \\
&SetNextTimeSlotBegValue&SET_NEXT_DOMAIN_BEG_VALUE&setNextTimeSlotBegValue& \\
&IntFutureValueByActivity&INT_FUTURE_VALUE_BY_ACTIVITY&intFutureValueByActivity& \\
&FloatFutureValueByActivity&FLOAT_FUTURE_VALUE_BY_ACTIVITY&floatFutureValueByActivity& \\
&IntFutureValueByActivityList&INT_FUTURE_VALUE_BY_ACTIVITY_LIST&intFutureValueByActivityList& \\
&FloatFutureValueByActivityList&FLOAT_FUTURE_VALUE_BY_ACTIVITY_LIST&floatFutureValueByActivityList& \\
&GetCriterionValue&GET_CRITERION_VALUE&getCriterionValue& \\
&UtilityByActivityList&UTILITY_BY_ACTIVITY_LIST&utilityByActivityList& \\
&BestActivitySetSelector&BEST_ACTIVITY_SET_SELECTOR&bestActivitySetSelector& \\
&LocalBestActivitySetSelector&LOCAL_BEST_ACTIVITY_SET_SELECTOR&localBestActivitySetSelector& \\
&CandidateActivitySetSortMethod&CANDIDATE_ACTIVITY_SET_SORT_METHOD&candidateActivitySetSortMethod& \\
&LocalCandidateActivitySetSortMethod&LOCAL_CANDIDATE_ACTIVITY_SET_SORT_METHOD&localCandidateActivitySetSortMethod& \\
\end{tabular}
 */
//@{

//=X===============X=============================X============================
// Activity/Activities
//............................................................................

class OpeningPredicate : public EntityPredicate
{
public:
   OpeningPredicate();
};

class ClosingPredicate : public EntityPredicate
{
public:
   ClosingPredicate();
};

class MinBegStatePredicate : public EntityPredicate
{
public:
   MinBegStatePredicate();
};

class MaxBegStatePredicate : public EntityPredicate
{
public:
   MaxBegStatePredicate();
};

class ModifyInOpenSituation : public EntityPredicate // + JPR 100820
{
public:
   ModifyInOpenSituation();
};

class UpdateReactivatedSon : public VoidEntityMethod
{
public:
   UpdateReactivatedSon();
};

class UpdateNextWhenSonOpen : public VoidEntityMethod
{
public:
   UpdateNextWhenSonOpen();
};

class UpdatePrevWhenSonOpen : public VoidEntityMethod
{
public:
   UpdatePrevWhenSonOpen();
};

class UpdateNextWhenSonClosed : public VoidEntityMethod
{
public:
   UpdateNextWhenSonClosed();
};

class UpdatePrevWhenSonClosed : public VoidEntityMethod
{
public:
   UpdatePrevWhenSonClosed();
};

class SetUserOperationName : public VoidEntityMethod
{
public:
   SetUserOperationName();
};

//=X===============X=============================X============================
// TransferInformation
//............................................................................

class SetUserTransferedIntQuantity : public IntEntityMethod
{
public:
   SetUserTransferedIntQuantity();
};

class SetUserTransferedFloatQuantity : public FloatEntityMethod
{
public:
   SetUserTransferedFloatQuantity();
};

//=X===============X=============================X============================
// Operation
//............................................................................

class StateTransitionProcedure : public EntityPredicate
{
public:
   StateTransitionProcedure();
};

class FeasibilityCondition : public EntityPredicate
{
public:
   FeasibilityCondition();
};

class SetUserUnitSpeed : public IntEntityMethod
{
public:
   SetUserUnitSpeed();
};

class SetUserIntQuantitySpeed : public IntEntityMethod
{
public:
   SetUserIntQuantitySpeed();
};

class SetUserFloatQuantitySpeed : public FloatEntityMethod
{
public:
   SetUserFloatQuantitySpeed();
};

class SetUserTimeSpeed : public FloatEntityMethod
{
public:
   SetUserTimeSpeed();
};

class SetUserStep : public IntEntityMethod
{
public:
   SetUserStep();
};

//=X===============X=============================X============================
// Resources
//............................................................................

class InvolvedEntities : public EntityTabEntityMethod
{
public:
   InvolvedEntities();
};

class CapacityOfferDemandComparison : public EntityPredicate
{
public:
   CapacityOfferDemandComparison();
};

//=X===============X=============================X============================
// Constraints
//............................................................................

class StateRelatedPredicate : public EntityPredicate
{
public:
   StateRelatedPredicate();
};

class HoldingCondition : public EntityPredicate
{
public:
   HoldingCondition();
};

class SetThisTimeSlotBegValue : public IntEntityMethod  // + JPR 100517
{
public:
   SetThisTimeSlotBegValue();
};

class SetThisTimeSlotEndValue : public IntEntityMethod  // + JPR 100517
{
public:
   SetThisTimeSlotEndValue();
};

class SetNextTimeSlotBegValue : public IntEntityMethod  // + JPR 100517
{
public:
   SetNextTimeSlotBegValue();
};

//=X===============X=============================X============================
// OperatedObjectSpec
//............................................................................


//=X===============X=============================X============================
// Goal
//............................................................................

class IntFutureValueByActivity : public IntEntityMethod
{
public:
   IntFutureValueByActivity();
};

class FloatFutureValueByActivity : public FloatEntityMethod
{
public:
   FloatFutureValueByActivity();
};

class IntFutureValueByActivityList : public IntEntityMethod
{
public:
   IntFutureValueByActivityList();
};

class FloatFutureValueByActivityList : public FloatEntityMethod
{
public:
   FloatFutureValueByActivityList();
};

//=X===============X=============================X============================
// Target_Goal + Anti_Target_Goal
//............................................................................


//=X===============X=============================X============================
// Criterion, Mono_Goal_Criterion
//............................................................................

class GetCriterionValue : public FloatEntityMethod
{
public:
   GetCriterionValue();
};

class UtilityByActivityList : public FloatEntityMethod
{
public:
   UtilityByActivityList();
};

//=X===============X=============================X============================
// System
//............................................................................

class BestActivitySetSelector : public EntityEntityMethod
{
public:
   BestActivitySetSelector();
};
class CandidateActivitySetSortMethod : public VoidEntityMethod
{
public:
   CandidateActivitySetSortMethod();
};

//=X===============X=============================X============================
// Strategy
//............................................................................

class LocalBestActivitySetSelector : public EntityEntityMethod
{
public:
   LocalBestActivitySetSelector();
};
class LocalCandidateActivitySetSortMethod : public VoidEntityMethod
{
public:
   LocalCandidateActivitySetSortMethod();
};

//=X===============X=============================X============================

//@}


#endif  // _C_COREMETHOD_HH
