//============================================================================
// FILE: C_CoreTypes.h 
//
// TYPE: C++
// DESCRIPTION: définition des types "moteur" et des variables globales
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
// REFERENCE : "Fondements ontologiques des systèmes pilotés"
//             R. Martin-Clouaire, J.-P. Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================#

#ifndef _C_CORETYPES_H
#define _C_CORETYPES_H

// SYSTEM INCLUDES
//

  using namespace std;
  #include <iostream>
  #include <cstdlib>
  #include <cstring>

// PROJECT INCLUDES
//

class NonPrimitiveActivity;
class PrimitiveActivity;
class ActivityRequired;
class ActivityBefore;
class ActivityBeforeAny;
class ActivityMeeting;
class ActivityMeetingAny;
class ActivityOverlapping;
class ActivityInclusion; 
class ActivityCostarting; 
class ActivityCoending; 
class ActivityEquality; 
class ActivityConjunction; 
class ActivityDisjunction; 
class ActivityIteration; 
class ActivityOptional; 
class Activity;
class Operation;
class UnitGranulatedOperation;
class QuantityGranulatedOperatio;
class TimeGranulatedOperation;
class OperationSpecification;
class OperationResourceSpecification;  // + JPR 100726
class OperatedObjectSpecification;     // modif JPR 100726 !
class PerformerSpecification;          // + JPR 100726
class ActivitiesResourcesBlockSet;
class ActivitiesResourcesBlock;
class InstructionSet;
class SingleInstructionSet;
class SinglePrimitiveInstructionSet;
//class SingleRequiredInstructionSet;
class MultipleInstructionSet;
class InstructionSetDisjunction;
class InstructionSetConjunction;
class SplittableInstructionSet;
class UnsplittableInstructionSet; 
class System;    
class ProductionSystem;
class OperatingSystem;
class ControlledSystem;
class Manager;
class InstructionAllocation;
class AllocatedResourceSet;
class AllocatedOperatedObjectSet;
class AllocatedOperationResourceSet;
class AllocatedPerformerSet;
class Resource;
class ResourceSet;
class ResourcePoolDisjunction;
class ResourcePoolConjunction;
class MultipleResourcePool;
class SimpleResourcePool;
class ActivitiesResourcesInconsistentCommitment;
class InconsistencyCondition;
class OccurrenceCrossDomain;
class Criterion;
class CriterionSet;
class Goal;
class ResourceCapacity;
class MultipleResourceCapacity;
class SingleStock;
class BufferizedDataFile; // + JPR 081027

#include "B_diese.h"

extern int (*f)();

// EXTERN_C void boot_bdiese _((CV* cv)); // à tester (031016)

//typedef Tableau<Activity*>              pActivityTab;
typedef Tableau<Activity*>             pActivityTab;  // 040608

/**@name Types prédéfinis, variable globales, constantes numériques et symboliques
\label{classeCdCoreTypes}
\begin{itemize} \item Directive d'inclusion : 
[P]
\#include C_CoreTypes.h
[/P]
\end{itemize}
*/
//@{

/**@name Variables globales pour la taille des énumérations 

Indiquent le plus grand numéro symbolique dans les énumérations {\em UserClassId}, {\em UserDescriptorId} \ldots .\\

Les valeurs numériques des constantes symboliques dans les énumérations complémentaires dans une sous-couche immédiate de \CD doivent être supérieures à ces valeurs. Ces valeurs doivent y être surchargées avec les plus grandes valeurs attribuées dans la sous-couche. Ces règles doivent être à nouveau appliquées pour une sous-couche encore inférieure. Ci-dessous, les valeurs attribuées dans la couche \CD.\\
       
[Y]g_BD_ReservedUserClassId[/Y]      : 195\\
[Y]g_BD_ReservedUserDescriptorId[/Y] : 230\\
[Y]g_BD_ReservedUserMethodId[/Y]     : 160\\
[Y]g_BD_ReservedUserArgumentId[/Y]   : 83\\
[Y]g_BD_ReservedUserProcessId[/Y]    : -9\\
[Y]g_BD_ReservedUserEventId[/Y]      :  3\\
[Y]g_BD_ReservedUserFileId[/Y]       : -9\\
*/ 
//@{
    // intentionnaly blank
//@}  // fin var globales taille enum

/**@name Variables globales liées aux arguments de la commande */
//@{

/* *@name Priorité des règles de supériorité entre jeux alternatifs d'activités

La méthode [Y]GetConsistentUpperSubsets[/Y] de la classe [C]InstructionSetDisjunction[/C] renvoie les plus grands sous-jeux consistants dans un ensemble de jeux alternatifs, en appliquant des "règles de supériorité" (voir Ontologie §5.5.2.2 et .3).\\

Règle 1 "SetSize" et "SetSize first" : le jeu S1 est supérieur à S2 s'il contient plus d'activités que S2. S2 est alors éliminé.\\\\
Règle 2 "MaxSpecAllocSize" et "MaxSpecAllocSize first" : le jeu S1 est supérieur à S2 si les deux contiennent la même activité avec une spécification de ressources de type MAX, et si S1 est allouée avec un ensemble plus grand de cette ressource que S2. S2 est alors éliminé.\\ 

L'utilisateur du simulateur peut choisir l'ordre dans lequel les règles seront appliquées, en donnant à la variable [Y]gUpperSubsetOption[/Y] une des valeurs suivantes :\\\\
C_SET_SIZE : Règle 1 seule\\\\
C_SET_MAX_SPEC_ALLOC_SIZE : Règle 2 seule\\\\
C_SET_SIZE_FIRST : Règle 1 avant Règle 2\\\\
C_SET_MAX_SPEC_ALLOC_SIZE_FIRST : Règle 2 avant Règle 1\\\\
C_NONE : Aucune règle n'est appliquée, aucun jeu n'est éliminé par l'application d'une de ces deux règles.\\\\

L'affectation se fait de la façon suivante :\\

[P]gUpperSubsetOption = C_SET_SIZE_FIRST;[/P]
ou\\
[P]gUpperSubsetOption = C_MAX_SPEC_ALLOC_SIZE_FIRST;[/P]

Par défaut, le moteur fixe la valeur à C_NONE. Elle peut être explicitement modifiée dans le code de l'application (comme ci-dessus), mais la manière la plus fréquente de le faire est d'utiliser l'option -USO dans la ligne de commande de la simulation :

[P]./main ... -USO setSize ...[/P]\\
ou\\
[P]./main ... -USO maxSpecAllocSize ...[/P]

*/
//@{
extern int gUpperSubsetOption; // obsolète 121223

enum C_UpperSubsetOption { // obsolète 121223
  C_NONE=0,C_SET_SIZE_FIRST=1, C_MAX_SPEC_ALLOC_SIZE_FIRST=2,C_SET_SIZE=3, C_MAX_SPEC_ALLOC_SIZE=4 
}; // modif JPR 101105
//@}

/**@name Borne supérieure de la taille d'un jeu d'activités

En préalable à l'allocation de ressources à un jeu d'activités candidat, le moteur réduit sa taille si le nombre d'activités primitives est supérieur à une valeur donnée. Par défaut, cette valeur est fixée dans \CD à l'infini, et aucune réduction n'est effectuée.\\

Cette borne supérieure peut être explicitement modifiée dans le code de l'application, de la manière suivante :

[P]gActSetSizeUpperBound = 4;[/P]

Mais la manière la plus fréquente de le faire est d'utiliser l'option -ActSet dans la ligne de commande de la simulation :

[P]./main ... -ActSet 4 ...[/P]

Cette réduction est parfois nécessaire pour éviter une explosion combinatoire des calculs, dont la complexité est exponentielle avec la taille du jeu. En effet, le jeu initial, s'il est inconsistant dans n'importe quelle situation d'allocation, est développé en un ensemble de sous-jeux possiblement consistants, auquels est appliqué, récursivement, le même algorithme d'allocation. De plus, les différentes situations d'allocations sont un "croisement" des différentes possibilités ouvertes pour chaque élément alloué dans l'activité : son objet opéré, les ressources propres de son opération, et son "performer".
*/
//@{
extern int gActSetSizeUpperBound;
//@}

/**@name Activation/désactivation de traces
*/
//@{
//--------------------------------------------------  // + JPR 100510
#ifndef _GTRACERESOURCEWARNINGS_H
#define _GTRACERESOURCEWARNINGS_H
/**@name Affichage de messages d'alerte sur les ressources

La valeur de la variable [Y]gTraceResourceWarnings[/Y] indique au moteur de simulation s'il doit ou non afficher certains messages d'alerte sur les ressources, notamment la découverte de leur indisponibilité.\\

Par défaut, la valeur est TRACE_RSC_WARN_OFF (pas de trace). La trace peut être activée par :\\

[PgTraceResourceWarnings] = TRACE_RSC_WARN_ON;[/P]

La manière la plus fréquente de le faire est d'utiliser l'option -RscWarn dans la ligne de commande de la simulation :

[P]./main ... -RscWarn ...[/P]
*/
//@{
enum boolTraceResourceWarnings {TRACE_RSC_WARN_OFF, TRACE_RSC_WARN_ON};

extern boolTraceResourceWarnings gTraceResourceWarnings;
//@}
#endif  // _GTRACERESOURCEWARNINGS_H
//--------------------------------------------------

//--------------------------------------------------  // + JPR 100803
#ifndef _GTRACEACTIVITYWARNINGS_H
#define _GTRACEACTIVITYWARNINGS_H
/**@name Affichage de messages d'alerte sur les activités

La valeur de la variable [Y]gTraceActivityWarnings[/Y] indique au moteur de simulation s'il doit ou non afficher certains messages d'alerte sur les activités. Les situations découvertes ne sont pas nécessairement signe ni génératrices d'erreur dans le modèle. Le développeur a toutefois généralement intérêt à leur trouver une justification valable.\\

Par défaut, la valeur est TRACE_ACT_WARN_OFF (pas de trace). La trace peut être activée par :\\

[P]gTraceActivityWarnings = TRACE_ACT_WARN_ON;[/P]

La manière la plus fréquente de le faire est d'utiliser l'option -ActWarn dans la ligne de commande de la simulation :

[P]./main ... -ActWarn ...[/P]
*/
//@{
enum boolTraceActivityWarnings {TRACE_ACT_WARN_OFF, TRACE_ACT_WARN_ON};

extern boolTraceActivityWarnings gTraceActivityWarnings;
//@}
#endif  // _GTRACEACTIVITYWARNINGS_H
//--------------------------------------------------

//--------------------------------------------------  // + JPR 100803
#ifndef _GTRACEACTIVITYLOGS_H
#define _GTRACEACTIVITYLOGS_H
/**@name Trace des tests effectués dans le cycle de vie des activités

La procédure [Y]UpdateSituation[/Y] de la classe [C]Activity[/C] opère les changement d'état des activités : de SLEEPING à CLOSED ou CANCELLED. Les tests effectués pour autoriser les changements d'état sont dans les méthodes du type [Y]CheckSonsIf<état>[/Y] et [Y]CheckIfSon<état>[/Y]. La trace de ces tests est obtenue en donnant la valeur TRACE_ACT_LOG_ON à la variable [Y]gTraceActivityLogs[/Y] (sa valeur par défaut est TRACE_ACT_LOG_OFF).

[P]gTraceActivityLogs = TRACE_ACT_LOG_ON;[/P]

La manière la plus fréquente de le faire est d'utiliser l'option -ActLog dans la ligne de commande de la simulation :

[P]./main ... -ActLog ...[/P]
*/
//@{
enum boolTraceActivityLogs {TRACE_ACT_LOG_OFF, TRACE_ACT_LOG_ON};

extern boolTraceActivityLogs gTraceActivityLogs;
//@}
#endif  // _GTRACEACTIVITYLOGS_H
//--------------------------------------------------


/**@name Les bornes de valeurs d'horloge pour les traces de procédures

Chacun de ces couples de valeurs encadrent la période dans laquelle une trace d'exécution est affichée sur la sortie standard pour quelques procédures importantes du moteur de simulation :\\
\\
\begin{tabular}{cclc}

{\tiny début} & {\tiny fin} & {\tiny procédure} & {\tiny option commande} \\
\hline

 {\tiny [Y]g_CD_logUPS_begClock[/Y]} & {\tiny [Y]g_CD_logUPS_endClock[/Y]} & {\tiny [Y]UpdateSituation[/Y]} & {\tiny -LogUPS} \\
 {\tiny [Y]g_CD_logMIL_begClock[/Y]} & {\tiny [Y]g_CD_logMIL_endClock[/Y]} & {\tiny [Y]MakeInstructionList[/Y]} & {\tiny -LogMIL}  \\
 {\tiny [Y]g_CD_logMILp_begClock[/Y]} & {\tiny [Y]g_CD_logMILp_endClock[/Y]} & {\tiny [Y]MakeInstructionList[/Y] (détails)} & {\tiny -LogMIL+}  \\
 {\tiny [Y]g_CD_logAIL_begClock[/Y]} & {\tiny [Y]g_CD_logAIL_endClock[/Y]} & {\tiny [Y]ActInstructionList[/Y]} & {\tiny -LogAIL}  \\
 {\tiny [Y]g_CD_logESA_begClock[/Y]} & {\tiny [Y]g_CD_logESA_endClock[/Y]} & {\tiny [Y]ExtendToSeveralAllocations[/Y]} &  {\tiny -LogESA}\\
 {\tiny [Y]g_CD_logESAp_begClock[/Y]} & {\tiny [Y]g_CD_logESAp_endClock[/Y]} & {\tiny [Y]ExtendToSeveralAllocations[/Y] (détails)} &  {\tiny -LogESA+}\\
\end{tabular}

Lorsque l'utilisateur du simulateur souhaite éditer ces traces, il fixe généralement les valeurs de ces variables dans la ligne de commande de la simulation, associées au mot-clé indiqué dans la dernière colonne du tableau ci-dessus. Ces valeurs peuvent aussi être affectées n'importe où dans le code du simulateur, pour un motif particulier.\\

Les valeurs par défaut de ces variables sont 0 pour la borne inférieure et -1 pour la borne supérieure de la période tracée. Ainsi, par défaut, il n'y a pas de trace.
*/

//@{
extern int g_CD_logUPS_begClock;  // + JPR 111222
extern int g_CD_logUPS_endClock;
extern int g_CD_logMIL_begClock;
extern int g_CD_logMIL_endClock;
extern int g_CD_logMILp_begClock;
extern int g_CD_logMILp_endClock;
extern int g_CD_logAIL_begClock;
extern int g_CD_logAIL_endClock;
extern int g_CD_logESA_begClock;
extern int g_CD_logESA_endClock;
extern int g_CD_logESAp_begClock;
extern int g_CD_logESAp_endClock;
//@}

extern Boolean g_CD_logUPS;
extern Boolean g_CD_logMIL;
extern Boolean g_CD_logMILp;
extern Boolean g_CD_logAIL;
extern Boolean g_CD_logESA;
extern Boolean g_CD_logESAp;

//@}
//@}  // fin var trace

/**@name Constantes numériques générales*/

//@{

/**@name Les valeurs par défaut des priorités

Les événements ci-dessous, prédéfinis dans \CD, ont une priorité établie par défaut, éventuellement surchargeable par le développeur.\\
\\
\begin{tabular}{ccc}

{\tiny événement} & {\tiny variable} & {\tiny valeur par défaut} \\
\hline

{\tiny [Y]ResourceImmobilizationEvent, ResourceMobilizationEvent[/Y]} & {\tiny [Y]PRIORITY_MOBIL[/Y]} & {\tiny 35} \\
{\tiny [Y]UpdateSituationEvent[/Y]} & {\tiny [Y]PRIORITY_US[/Y]} & {\tiny 40} \\
{\tiny [Y]MakeInstructionListEvent[/Y]} & {\tiny [Y]PRIORITY_MAKE[/Y]} & {\tiny 45} \\
{\tiny [Y]ActInstructionListEvent[/Y]} & {\tiny [Y]PRIORITY_ACT[/Y]} & {\tiny 50} \\
{\tiny [Y]ProceedOperationEvent[/Y] (opérations permanentes)} & {\tiny [Y]PRIORITY_PROCEED_OTPERM[/Y]} & {\tiny 59} \\
{\tiny [Y]ProceedOperationEvent[/Y]} & {\tiny [Y]PRIORITY_PROCEED_OT[/Y]} & {\tiny 60} \\
\end{tabular}

Lors de l'identification d'une classe d'événements dans le domaine d'application, le développeur doit attribuer une priorité (dans l'intervalle [0 99], 0 étant la plus forte priorité) en connaissance des valeurs dans le tableau ci-dessus. On rappelle que le moteur de simulation utilise cette priorité pour ordonner dans l'agenda deux événements intervenant à la même valeur de l'horlode de la simulation.
*/
//@{
extern short PRIORITY_MOBIL;
extern short PRIORITY_US;
extern short PRIORITY_MAKE;
extern short PRIORITY_ACT;
extern short PRIORITY_CHECK_TRAJECTORY;
extern short PRIORITY_PROCEED_OTPERM;
extern short PRIORITY_PROCEED_OT;
//@}

/**@name Degré de précision pour l'égalité des flottants

   Le développeur peut fixer la valeur de la demi-longueur de l'intervalle autour d'un nombre flottant X, à l'intérieur duquel tout nombre x est considéré égal à X. Par défaut, \CD fixe sa valeur à 0.00001. Elle peut être modifiée dans le code de l'application : \\

[P]gFloatingPointEqualityTolerance = 0.05;[/P]

Le moteur de \CD exploite cette valeur pour tester la complétude à 1.0 de la réalisation d'une opération.\\

Pour sa part, le développeur peut, non seulement dimensionner l'intervalle en fonction de son application, mais exploiter lui-même cette variable pour effectuer, ici ou là dans son code, les tests de comparaison entre nombres flottants.
*/

//@{
extern float gFloatingPointEqualityTolerance;  // + JPR 110414
//@}

//@}  // fin Constantes numériques générales


/**@name Constantes symboliques ({\em enum}) générales à tous les domaines

Les constantes symboliques, incluses dans les {\em énumérations}\index{énumérations} ci-dessus, sont en fait des nombres entiers positifs associés à des symboles (un par nombre). Il est alors équivalent de faire référence à la valeur numérique où à sa traduction symbolique. Par exemple, les deux messages suivants sont équivalents : \\

[P]pAct->SetIntVarValue(SITUATION, 3)[/P]\\ 
et \\
[P]pAct->SetIntVarValue(SITUATION, OPEN)[/P]\\

(voir l'énumération 'états d'activités et d'opérations').
 */ 
//@{

//----------------------------------------------------------------------------
//      Symboles d'état des activités et des operations
//----------------------------------------------------------------------------

/**@name C_StateId : états d'activités et d'opérations

{\large {\tt Activités}} (classe [C]Activity[/C]) : \\

SLEEPING=1, WAITING, OPEN, IN_COURSE, CLOSED, CANCELLED=6, NO_LONGER_OPENABLE=7, \\

{\large {\tt Opérations}} (classe [C]Operation[/C]) : \\

EXECUTING=11, SUSPENDED, TO_STOP, TERMINATED=14, CANCELLED=6. \\

Enumération exhaustive des situations possibles pour les activités et les opérations.\\

De manière générale, les changements de valeurs d'état sont effectués dans le code du moteur de simulation : le code de l'application ne comporte normalement pas d'instruction tette que :\\

[P]pAct->SetIntVarValue(SITUATION, OPEN)[/P]\\

Seul, le changement d'état d'une opération de la valeur EXECUTING vers la valeur TO_STOP peut éventuellement se trouver dans le code de l'application, pour stopper la progression de l'effet de l'opération (à condition qu'un seuil donné soit dépassé).
*/
//@{
enum C_StateId{
SLEEPING=1, WAITING, OPEN=3, IN_COURSE, CLOSED, CANCELLED, NO_LONGER_OPENABLE, EXECUTING=11, SUSPENDED, TO_STOP, TERMINATED
};
//@}

/**@name C_ContinuationMode : modes de continuation des activités

NEW_ENTITIES=1, SAME_PREVIOUS_ENTITIES=2, SAME_FIRST_ENTITIES=3\index{ContinuationMode}\\

Liste exhaustive des modes de continuation d'une activité, lorsque celle-ci est reprise après que son opération a été interrompue pendant un délai. Ce sont les valeurs possibles du descripteur [C]ContinuationMode[/C], des classes [C]OperatedObjectSpecification[/C] et [C]PerformerSpecification[/C].\\

Le mode de continuation spécifie que :\\
- l'activité porte ou non sur les mêmes entités que celles qui en étaient déjà l'objet en l'instant de l'interruption. \\
- l'activité est exécutée par les mêmes "performers" ou non que ceux qui en étaient déjà les exécuteurs en l'instant de l'interruption. \\

Seulement si continuation est NEW_ENTITIES, on opère un nouveau développement de la spécification d'objets opérés ou du "performer".\\

Si la valeur est SAME_FIRST_ENTITIES, la spécification requiert d'opérer sur les entités qui étaient présentes dans le résultat de la {\em première} expansion (au début de la mise en oeuvre de l'activité) et qui sont encore dans le résultat de l'expansion en l'instant de la reprise. Pour les "performers", la spécification requiert d'opérer avec les "performers" qui étaient présents dans le résultat de la première expansion (au début de la mise en oeuvre de l'activité) et qui sont encore dans le résultat de l'expansion en l'instant de la reprise.\\

Si la valeur est SAME_PREVIOUS_ENTITIES, la spécification requiert d'opérer sur les entités qui étaient présentes dans le résultat de la {\em précédente} expansion (avant l'interruption) et qui sont encore dans le résultat de l'expansion en l'instant de la reprise. Pour les "performers", la spécification requiert d'opérer avec les "performers" qui étaient présents dans le résultat de la {\em précédente expansion} (avant l'interruption) et qui sont encore dans le résultat de l'expansion en l'instant de la reprise.\\

Si la valeur est EXACTLY_SAME, la spécification de "performer" requiert d'opérer avec les tous les "performers" qui étaient présents dans le résultat de la {\em précédente} expansion.\\

Pour fixer cette valeur dans le code de la base de connaissances, on écrira, typiquement dans le constructeur d'une spécification :\\

[P]PerformerSpecification* pPerfSpec;\\
pPerfSpec->SetIntConstValue(CONTINUATION_MODE, SAME_PREVIOUS_ENTITIES)[/P]\\

Pour fixer cette valeur dans un fichier de données externe, on écrira :\\

\begin{verbatim}
[P]+ I performerSpecification pPerfSpec
    ...
    continuationMode = 3;  // SAME_PREVIOUS_ENTITIES
    ...
;[/P]
\end{verbatim}
*/
//@{
enum C_ContinuationMode {
   NEW_ENTITIES=1,
   SAME_PREVIOUS_ENTITIES=2,
   SAME_FIRST_ENTITIES=3,
   EXACTLY_SAME=4             // + JPR 110111
};
//@}

/* *@name C_DistributionMode : modes de distribution dans une activité de transfert

TOP_TO_TOP=1, ONE_TO_ONE=2\index{DistributionMode}\\

Liste exhaustive des modes de distribution dans une activité de transfert
!! non implémenté dans la version courante !!
(voir ontologie 4.6)
*/
//@{
enum C_DistributionMode {
   TOP_TO_TOP=1,
   ONE_TO_ONE=2
};
//@}

/**@name C_ProportionalityMode : proportionalité de l'usage des ressources

NON_PROPORTIONAL=1, PROPORTIONAL=2\index{ProportionalityMode}\\

Le moteur de simulation développe la spécification de ressources propres d'une opération (classe [C]OperationResourceSpecification[/C]) après avoir alloué l'opérateur (i.e. les "performers") de l'activité (par le développement d'une instance de la classe [C]PerformerSpecification[/C]). \\

Dans le mode NON_PROPORTIONAL, l'ensemble de ressources issues de l'expansion de l'ensemble d'entités-ressources est requis tel quel, quelle que soit la puissance de l'opérateur.\\

Dans le mode PROPORTIONAL, l'ensemble de ressources issues de l'expansion de l'ensemble d'entités-ressources est requis pour chaque unité de la puissance de l'opérateur.\\

Ainsi, par exemple, si l'opération OT de l'activité primitive A requiert 1 unité d'outil avec le mode PROPORTIONAL, et si 2 unités de main d'oeuvre sont allouées à A, alors OT doit se voir allouer 2 unités d'outil.\\

Pour fixer cette valeur dans le code de la base de connaissances, on écrira, typiquement dans le constructeur d'une spécification :\\

[P]OperationResourceSpecification* pRscSpec;\\
pRscSpec->SetIntConstValue(PROPORTIONALITY_MODE, PROPORTIONAL)[/P]\\

Pour fixer cette valeur dans un fichier de données externe, on écrira :\\

\begin{verbatim}
[P]+ I operationResourceSpecification pRscSpec
    ...
    proportionalityMode = 2;  // PROPORTIONAL (*)
    ...
;[/P]
\end{verbatim}
On peut aussi écrire :
\begin{verbatim}
[P]    proportionalityMode = PROPORTIONAL;[/P]
\end{verbatim}
*/
//@{
enum C_ProportionalityMode {
   NON_PROPORTIONAL=1,
   PROPORTIONAL=2
};
//@}

/**@name C_SelectionMode : modes de sélection des ressources spécifiées

ANYONE=1, MAX=2, ALL=3, DISJ=4, SETS=5\index{SelectionMode}\\

Pour chaque type de ressources à allouer (objets opéré, ressource propres d'opération, opérateur), le moteur développe une "spécification de ressources" (attachée à l'activité ou à l'opération selon le cas). Dans chaque cas, ce développement génère une liste "à plat" d'instances de ressources d'une classe donnée de symbole {\em id}. Une seconde phase consiste à choisir dans cette liste de ressources, appelée ici {\em L}, de taille {\em s}, une ou plusieurs manières de les allouer. C'est le "mode de sélection", sujet de la présente énumération, qui détermine le résultat. \\

Le mode est la valeur du descripteur [C]SelectorFctId[/C] de la classe de spécification (classe [C]OperatedObjectSpecification[/C] ou [C]OperationResourceSpecification[/C] ou enfin [C]PerformerSpecification[/C]). Cette valeur est fixée par le développeur dans le constructeur de la classe, ou bien par l'utilisateur du simulateur dans un des fichiers d'entrée.\\

Avec le mode ANYONE, complété par un nombre entier {\em n}, on alloue les {\em n} premières ressources de {\em L} trouvées disponibles.\\

Avec le mode MAX, on alloue toutes les ressources de {\em L} trouvées disponibles. \\

Avec le mode ALL, on alloue toutes les ressources de {\em L}, ou bien on conclue à l'échec de l'allocation si une d'entre elles est trouvée indisponible.\\

Avec le mode DISJ, on génère une liste de {\em s} listes, chacune ne comprenant qu'une ressource disponible de {\em L}, différente à chaque fois.\\

Avec le mode SETS, complété par un nombre entier {\em n}, on génère une liste de {\em p} listes, chacune de {\em n} ressources disponibles. {\em p} est le nombre de manières d'extraire {\em n} ressources des {\em k} sous-classes terminales de celle de symbole {\em id}. \\

Se reporter au document "Ontologie des systèmes pilotés", pour une présentation complète et illustrée d'exemples.

Pour fixer ces valeurs dans le code de la base de connaissances, on écrira, typiquement dans le constructeur d'une spécification :\\

[P]OperatedObjectSpecification* pOOSpec;\\
pOOSpec->SetIntVarValue(ENTITY_SPEC_REFERENCE_CLASS_ID, MY_TOOL);
pOOSpec->SetIntConstValue(SELECTOR_FCT_ID, ANYONE);\\
pOOSpec->SetIntConstValue(SELECTOR_FCT_ARG, 2);
[/P]\\

Pour fixer cette valeur dans un fichier de données externe, on écrira :\\

\begin{verbatim}
[P]+ I operatedObjectSpecification pOOSpec
    ...
    entitySpecReferenceClassId = 267; // MY_TOOL
    selectorFctId = 1;                // ANYONE
    selectorFctArg = 2;
    ...
;[/P]
\end{verbatim}
*/
//@{
enum C_SelectionMode {
   ANYONE=1, MAX, ALL, DISJ, SETS
};

enum C_UpDown {
FROM_DOWN=0, FROM_UP=1, FROM_NULL
};

enum C_SetType {
C_DISJ=0, C_CONJ, C_SPLIT, C_UNSPLIT, C_PRIM, C_REQ
};

enum C_StockAccesMode {  // obsolète ?
C_IN=0, C_OUT
};

enum C_QuantityId {
C_ELEMENTS=0
};

enum C_QuantityOption {  // obsolète ?
C_WHOLE=0, C_PART
};

enum C_GoalId {
C_TARGET=0, C_ANTI_TARGET
};

enum C_ActivityActionMode {
   C_TAKE_MIN=1, C_TAKE_MEAN, C_TAKE_MAX
};
//@}

         /**@name C_BinaryOperator : opérateurs de comparaison binaire

C_LT=1, C_LE, C_EQ, C_NEQ, C_GE, C_GT\index{BinaryOperator}\\

Le développeur et/ou l'utilisateur peuvent spécifier des situations illégales d'allocation de ressources (parce qu'impossibles, interdites, etc.). Une situation illégale est "codée" sous la forme d'un triplet <classe de ressources, opérateur, seuil> (c'est une instance de la classe [C]OccurrenceDomain[/C]).\\

Par exemple la spécification <MY_TOOL, C_GE, 2> interdit l'allocation de plus d'une instance de la classe [C]MyTool[/C]. Le moteur de simulation teste la relation 'nombre d'instances alloués operateur seuil'. Si on tente d'allouer 2 instances, la relation est vérifiée et la situation est déclarée inacceptable.\\ 

Ci-dessus est la liste exhaustive des opérateurs de comparaison binaire utilisables dans ces spécifications de situation illégales d'engagement de ressources.\\ 

Programmatiquement, la spécification est réalisée par des lignes telles que :
\begin{verbatim}
[P]OccurrenceDomain* pOD = new OccurrenceDomain();
pOD->SetIntConstValue(ENTITY_ID_ATTRIBUTE, MY_TOOL);
pOD->SetIntConstValue(BINARY_OPERATOR, C_GE);
pOD->SetIntConstValue(ACCEPTABILITTHRESHOLD, 2);[/P]
\end{verbatim}
*/
//@{
enum C_BinaryOperator {
  C_LT=1, C_LE, C_EQ, C_NEQ, C_GE, C_GT
};
//@}


//----------------------------------------------------------------------------
//      Symboles des modes de progression d'opération
//----------------------------------------------------------------------------

/**@name C_ProgressionModeId : modes de progression des opérations\\

Les opérations (classe [C]Operation[/C]) se distinguent, entre autres choses par le mode de progression de l'effet. Les trois modes de progression sont décrits dans les pages dédiées aux sous-classes qui les portent.

\begin{tabular}{cccc}
{\tiny mode} & {\tiny entier associé} & {\tiny classes porteuses} \\ \hline
{\tiny C_UNIT} & {\tiny 0} & {\tiny UnitGranulatedOperation UnitGranulatedTransferOperation} \\
{\tiny C_INT_QUANTITY} & {\tiny 1} & {\tiny IntQuantityGranulatedOperation IntQuantityGranulatedTransferOperation} \\
{\tiny C_FLOAT_QUANTITY} & {\tiny 2} & {\tiny FloatQuantityGranulatedOperation FloatQuantityGranulatedTransferOperation} \\
{\tiny C_TIME} & {\tiny 3} & {\tiny TimeGranulatedOperation}
\end{tabular}

Une de ces valeurs est donnée, par construction, au descripteur [C]ProgressionMode[/C] des instances de la classe [C]Operation[/C].

*/
//@{
enum C_ProgressionModeId {
C_UNIT=0, C_INT_QUANTITY, C_FLOAT_QUANTITY, C_TIME
};
//@}

/**@name Symboles des classes prédéfinies 

Chaque classe prédéfinie dans \CD est dotée d'un symbole caractéristique. Ce symbole est associé bijectivement à une valeur entière dans une {\em énumération} C++. Il en est de même pour les classes définies dans l'application, et qui héritent nécessairement des classes prédéfinies dans \BD ou \CD. \\

Il est important de noter ou de rappeler que tous ces symboles doivent être différents (et donc aussi les entiers associés) entre toutes les classes d'entités. Il en est de même entre toutes les classes d'événements, de processus, de descripteurs, de méthodes et d'arguments.\\

Pour les entités, les symboles doivent donc être tous différents entre toutes les classes d'activités, d'opérations ou d'une autre sous-classe d'entités. On constarera en effet que les entiers pour les activités sont entre 11 et 26, les entiers pour les opérations sont entre 71 et 77, etc.\\

Au plan pratique, les valeurs numériques des constantes symboliques dans une base de connaissances développées sous \CD doivent être supérieures à la plus grande valeur déjà en usage dans \CD. Ces valeurs sont indiquées dans la page 'Variables globales pour la taille des énumérations'.
       
*/
//@{

//----------------------------------------------------------------------------
//      Symboles des classes d'événements
//----------------------------------------------------------------------------

/**@name Symboles des classes d'événements\\
\index{C_EventClassId (énumération)}

UPDATE_SITUATION_EVENT=11,\\ 
MAKE_INSTRUCTION_LIST_EVENT,\\ 
ACT_INSTRUCTION_LIST_EVENT,\\ 
PROCEED_OPERATION_EVENT=14,\\

RESOURCE_IMMOBILIZATION_EVENT=15,\\ 
RESOURCE_MOBILIZATION_EVENT=16\\

Enumération exhaustive des symboles des classes d'événements prédéfinis.\\\index{événements!symboles de classe}

Les conséquences de l'occurrence de ces événements (qui sont le motif de leur pargrammation dans l'agenda de la simulation) sont précisées dans la page dédiée à chaque classe.\\

Les événements dont le symbole est MAKE_INSTRUCTION_LIST_EVENT ou ACT_INSTRUCTION_LIST_EVENT ou PROCEED_OPERATION_EVENT sont normalement programmés et gérés exclusivement par le moteur de simulation. Le code spécifique d'une application peut éventuellement connaître l'existence dans l'agenda de tels événements, mais il ne doit pas intervenir sur cette programmation.\\

Dans un modèle comportant un plan d'activités, au moins un événement instance directe ou indirecte de la classe de symbole UPDATE_SITUATION_EVENT ([C]UpdateSituationEvent[/C]) doit figurer dans l'agenda.\\

En fonction du besoin, le développeur peut créer des sous-classes particulières des classes de symbole RESOURCE_IMMOBILIZATION_EVENT ou RESOURCE_MOBILIZATION_EVENT (resp. [C]ResourceImmobilizationEvent[/C] et [C]ResourceMobilizationEvent[/C]), ou bien installer dans l'agenda des instances directes de ces classes.
*/
//----------------------------------------------------------------------------
// cf. g_BD_ReservedUserEventId  ; mettre à jour g_BD_ReservedUserEventId
//@{
enum C_EventClassId {UPDATE_SITUATION_EVENT=11, MAKE_INSTRUCTION_LIST_EVENT, ACT_INSTRUCTION_LIST_EVENT, PROCEED_OPERATION_EVENT,
RESOURCE_IMMOBILIZATION_EVENT, RESOURCE_MOBILIZATION_EVENT=16
};
//@}

//----------------------------------------------------------------------------
//      Symboles des classes de processus
//----------------------------------------------------------------------------

/**@name Symboles des classes des processus
\index{C_ProcessClassId (énumération)}
\begin{tabular}{cccc}
{\tiny symbole de classe} & {\tiny entier associé} & {\tiny classe } & {\tiny catégorie}\\ \hline
{\tiny UPDATE_SITUATION_PROCESS} & {\tiny 2} & {\tiny UpdateSituationProcess} & {\tiny Discrete}\\
{\tiny MAKE_INSTRUCTION_LIST_PROCESS} & {\tiny 3} & {\tiny MakeInstructionListProcess} & {\tiny Discrete}\\
{\tiny ACT_INSTRUCTION_LIST_PROCESS} & {\tiny 5} & {\tiny ActInstructionListProcess} & {\tiny Discrete}\\
{\tiny PROCEED_OPERATION_PROCESS} & {\tiny 7} & {\tiny ProceedOperationProcess} & {\tiny Continuous}\\
{\tiny AUGMENT_OPERATION_PERCENT_ACHIEVED_PROCESS} & {\tiny 8} & {\tiny AugmentOperationPercentAchievedProcess} & {\tiny Discrete}\\
{\tiny RESOURCE_IMMOBILIZATION_PROCESS} & {\tiny 9} & {\tiny ResourceImmobilizationProcess} & {\tiny Discrete}\\
{\tiny RESOURCE_MOBILIZATION_PROCESS} & {\tiny 10} & {\tiny ResourceMobilizationProcess} & {\tiny Discrete}\\
{\tiny READ_BUFFERIZED_DATA_FILE_PROCESS} & {\tiny 15} & {\tiny ReadSequentialDataFileProcess} & {\tiny Discrete}\\
\end{tabular}

Enumération exhaustive des symboles de classe des processus prédéfinis.\\ \index{processus!symboles de classe}

Le commentaire fait à la page des événements prédéfinis s'applique ici encore.
*/
//----------------------------------------------------------------------------
// cf. g_BD_ReservedUserProcessId  ; mettre à jour g_BD_ReservedUserProcessId
//@{
enum C_ProcessClassId {CHECK_TRAJECTORY=1, UPDATE_SITUATION_PROCESS, MAKE_INSTRUCTION_LIST_PROCESS, MAKE_INSTRUCTION_LIST_PROCESS2, ACT_INSTRUCTION_LIST_PROCESS, REACT_TO_ALARM, PROCEED_OPERATION_PROCESS, AUGMENT_OPERATION_PERCENT_ACHIEVED_PROCESS,
RESOURCE_IMMOBILIZATION_PROCESS=9, RESOURCE_MOBILIZATION_PROCESS,
READ_BUFFERIZED_DATA_FILE_PROCESS=15      // + JPR 081027

		   //
		   // (090806) READ_BUFFERIZED_DATA_FILE_PROCESS est passé de 9 à 10 !!
		   // (100531) READ_BUFFERIZED_DATA_FILE_PROCESS est passé de 10 à 15 !!
		   //

		   // CHECK_TRAJECTORY : non implémenté
		   // REACT_TO_ALARM : non implémenté
};
//@}


//----------------------------------------------------------------------------
//      Symboles des classes d'entités
//----------------------------------------------------------------------------

/**@name Symboles des classes d'entités

Les entiers 0, 1 et 2 sont associés aux symboles des classes d'entités prédéfinies dans DIESE ([C]BasicEntity[/C], [C]DescriptedEntity[/C] et [C]Entity[/C].\\

Généralement, les classes dont les symboles sont listés ici sont mères ou ancêtres des classes identifiées par le développeur dans son domaine d'application.\\

Certaines classes, au contraire, sont instanciées et gérées exclusivement par le moteur de simulation. Leurs symboles sont dévoilés ici à toutes fins utile. Par exemple, le code de l'application peut informer une règle de décision avec le contenu d'une allocation de ressource, et accéder pour cela, toujours par exemple, au composant de symbole ALLOCATED_PERFORMER_SET de la classe [C]InstructionAllocation[/C] (de symbole INSTRUCTION_ALLOCATION).

{\large {\tt Activités :}} \\

ACTIVITY=11, \\ 
PRIMITIVE_ACTIVITY, TRANSFER_ACTIVITY, \\
NON_PRIMITIVE_ACTIVITY, \\
ACTIVITY_BEFORE, ACTIVITY_MEETING, \\
ACTIVITY_BEFORE_ANY, ACTIVITY_MEETING_ANY, \\
ACTIVITY_OVERLAPPING, ACTIVITY_INCLUSION, \\
ACTIVITY_EQUALITY, ACTIVITY_COSTARTING, ACTIVITY_COENDING, \\
ACTIVITY_DISJUNCTION, ACTIVITY_CONJUNCTION, \\
ACTIVITY_ITERATION, ACTIVITY_OPTIONAL,  \\
ACTIVITY_REQUIRED=28
{\large {\tt Jeux d'activités :}} \\

ACTIVITY_SET=30, 
INSTRUCTION_SET, SINGLE_INSTRUCTION_SET, SINGLE_PRIMITIVE_INSTRUCTION_SET, SINGLE_REQUIRED_INSTRUCTION_SET, 
MULTIPLE_INSTRUCTION_SET, INSTRUCTION_SET_DISJUNCTION, INSTRUCTION_SET_CONJUNCTION, SPLITTABLE_INSTRUCTION_SET, UNSPLITTABLE_INSTRUCTION_SET=39,\\

{\large {\tt Allocations de ressources :}} \\

INSTRUCTION_ALLOCATION=41, 
INSTRUCTION_SET_ALLOCATION, 
COENGAGEMENT_ITEM, USAGE_ITEM, RESOURCE_USAGE_ITEM, OPERATION_USAGE_ITEM, 
RESOURCE_SET, RESOURCE_POOL_DISJUNCTION, RESOURCE_POOL_CONJUNCTION, MULTIPLE_RESOURCE_POOL, SIMPLE_RESOURCE_POOL, 
ALLOCATED_RESOURCE_SET, ALLOCATED_OPERATED_OBJECT_SET, ALLOCATED_DESTINATION_OBJECT_SET, ALLOCATED_OPERATION_RESOURCE_SET, ALLOCATED_PERFORMER_SET, ALLOCATED_OPERATION_SET=57,\\

{\large {\tt Spécifications :}} \\

OPERATED_OBJECT_SPECIFICATION=61, OPERATION_SPECIFICATION, PERFORMER_SPECIFICATION, OPERATION_RESOURCE_SPECIFICATION, 
TRANSFER_INFORMATION=65,\\

{\large {\tt Opérations :}} \\

OPERATION=71, \\ 
UNIT_GRANULATED_OPERATION, INT_QUANTITY_GRANULATED_OPERATION, FLOAT_QUANTITY_GRANULATED_OPERATION, TIME_GRANULATED_OPERATION,
UNIT_GRANULATED_TRANSFER_OPERATION, INT_QUANTITY_GRANULATED_TRANSFER_OPERATION, FLOAT_QUANTITY_GRANULATED_TRANSFER_OPERATION=78,\\

{\large {\tt Ressources :}} \\

RESOURCE=81,  
HOMOGENEOUS_AGGREGATED_RESOURCE,  
HOMOGENEOUS_AGGREGATED_DISCRETE_STATE_RESOURCE,
HOMOGENEOUS_AGGREGATED_DISCRETE_STATE_REUSABLE_RESOURCE,
HOMOGENEOUS_AGGREGATED_DISCRETE_STATE_CONSUMABLE_RESOURCE,
HOMOGENEOUS_AGGREGATED_CAPACITATED_RESOURCE,
HOMOGENEOUS_AGGREGATED_CAPACITATED_REUSABLE_RESOURCE,
HOMOGENEOUS_AGGREGATED_CAPACITATED_CONSUMABLE_RESOURCE,
HOMOGENEOUS_AGGREGATED_PERFORMER, HOMOGENEOUS_LABOR_TEAM=90,

HETEROGENEOUS_AGGREGATED_RESOURCE=91, 
HETEROGENEOUS_AGGREGATED_PERFORMER, HETEROGENEOUS_LABOR_TEAM=93\\

SINGLE_RESOURCE=94,\\
SINGLE_DISCRETE_STATE_RESOURCE, SINGLE_DISCRETE_STATE_REUSABLE_RESOURCE, SINGLE_DISCRETE_STATE_CONSUMABLE_RESOURCE, \\
SINGLE_CAPACITATED_RESOURCE, SINGLE_CAPACITATED_REUSABLE_RESOURCE, SINGLE_CAPACITATED_CONSUMABLE_RESOURCE=100, \\ 

SINGLE_PERFORMER=101, SINGLE_WORKER, 
SINGLE_TOOL, 
SINGLE_STOCK,
OPERATED_OBJECT, OPERATED_LOCATION, OPERATED_POPULATION, OPERATED_RESOURCE, NON_SPECIAL_OPERATED_OBJECT=109,\\

{\large {\tt Contraintes sur ressources :}} \\

MULTIPLE_RESOURCE_CAPACITY=112, \\
RESOURCE_CAPACITY, INTEGER_RESOURCE_CAPACITY, FLOAT_RESOURCE_CAPACITY, \\
STOCK_CAPACITY, INTEGER_STOCK_CAPACITY, FLOAT_STOCK_CAPACITY=118,\\
  
AVAILABILITY_CONSTRAINT=125, TIME_AVAILABILITY_CONSTRAINT, STATE_AVAILABILITY_CONSTRAINT, CAPACITY_AVAILABILITY_CONSTRAINT, STANDARD_CAPACITY_AVAILABILITY_CONSTRAINT=129,\\

AVAILABILITY_TIME_SLOT=130, RECURRENT_AVAILABILITY_TIMETABLE=131,\\

INCONSISTENCY_CONDITION=132, RESOURCE_SHARING_VIOLATION_CONDITION, ACTIVITY_INCONSISTENCY_CONDITION, ACTIVITIES_RESOURCES_INCONSISTENT_COMMITMENT, OCCURRENCE_DOMAIN, OCCURRENCE_CROSS_DOMAIN=137,\\

{\large {\tt Ensembles structurés d'individus :}} \\

SETTLEMENT=141, MULTIPLE_SETTLEMENT, POPULATION, MULTIPLE_POPULATION, SIMPLE_POPULATION, SIMPLE_HOMOGENEOUS_POPULATION, SIMPLE_HETEROGENEOUS_POPULATION=147,\\

{\large {\tt Structuration de l'espace :}} \\

LOCATION=151,\\

{\large {\tt Systèmes :}} \\

SYSTEM=161, SET_SYSTEM, MANAGER, OPERATING_SYSTEM, CONTROLLED_SYSTEM, PRODUCTION_SYSTEM=166,\\

{\large {\tt Stratégies et composants :}} \\

STRATEGY=171, ACTIVITIES_RESOURCES_BLOCK_SET, ACTIVITIES_RESOURCES_BLOCK=173\\

{\large {\tt Buts et critères:} }\\

GOAL=181, INT_GOAL, FLOAT_GOAL, INT_TARGET, FLOAT_TARGET, INT_ANTI_TARGET, FLOAT_ANTI_TARGET=187,

CRITERION=188, CRITERION_SET, GOAL_CRITERION, UTILITY_CRITERION, TARGET_UTILITY_CRITERION, ANTI_TARGET_UTILITY_CRITERION, GLOBAL_TARGET_CRITERION, GLOBAL_ANTI_TARGET_CRITERION=195
 */
//@{

// cf. g_BD_ReservedUserClassId  ; mettre à jour g_BD_ReservedUserClassId
enum C_EntityClassId {
ACTIVITY=11, 
PRIMITIVE_ACTIVITY, TRANSFER_ACTIVITY,
NON_PRIMITIVE_ACTIVITY,
ACTIVITY_BEFORE, ACTIVITY_MEETING,
ACTIVITY_BEFORE_ANY, ACTIVITY_MEETING_ANY,
ACTIVITY_OVERLAPPING, ACTIVITY_INCLUSION,
ACTIVITY_EQUALITY, ACTIVITY_COSTARTING, ACTIVITY_COENDING,
ACTIVITY_DISJUNCTION, ACTIVITY_CONJUNCTION,
ACTIVITY_ITERATION, ACTIVITY_OPTIONAL, 
ACTIVITY_REQUIRED=28,

ACTIVITY_SET=30, INSTRUCTION_SET, SINGLE_INSTRUCTION_SET, SINGLE_PRIMITIVE_INSTRUCTION_SET, SINGLE_REQUIRED_INSTRUCTION_SET, MULTIPLE_INSTRUCTION_SET, INSTRUCTION_SET_DISJUNCTION, INSTRUCTION_SET_CONJUNCTION, SPLITTABLE_INSTRUCTION_SET, UNSPLITTABLE_INSTRUCTION_SET,

INSTRUCTION_ALLOCATION=41, INSTRUCTION_SET_ALLOCATION, COENGAGEMENT_ITEM, USAGE_ITEM, RESOURCE_USAGE_ITEM, OPERATION_USAGE_ITEM, 
RESOURCE_SET, RESOURCE_POOL_DISJUNCTION, RESOURCE_POOL_CONJUNCTION, MULTIPLE_RESOURCE_POOL, SIMPLE_RESOURCE_POOL, ALLOCATED_RESOURCE_SET, ALLOCATED_OPERATED_OBJECT_SET, ALLOCATED_DESTINATION_OBJECT_SET, ALLOCATED_OPERATION_RESOURCE_SET, ALLOCATED_PERFORMER_SET, ALLOCATED_OPERATION_SET, 

OPERATED_OBJECT_SPECIFICATION=61, OPERATION_SPECIFICATION, PERFORMER_SPECIFICATION, OPERATION_RESOURCE_SPECIFICATION, TRANSFER_INFORMATION,
 
OPERATION=71, 
UNIT_GRANULATED_OPERATION, INT_QUANTITY_GRANULATED_OPERATION, FLOAT_QUANTITY_GRANULATED_OPERATION, TIME_GRANULATED_OPERATION,
UNIT_GRANULATED_TRANSFER_OPERATION, INT_QUANTITY_GRANULATED_TRANSFER_OPERATION, FLOAT_QUANTITY_GRANULATED_TRANSFER_OPERATION,

RESOURCE=81,  
HOMOGENEOUS_AGGREGATED_RESOURCE,  
HOMOGENEOUS_AGGREGATED_DISCRETE_STATE_RESOURCE,
HOMOGENEOUS_AGGREGATED_DISCRETE_STATE_REUSABLE_RESOURCE,
HOMOGENEOUS_AGGREGATED_DISCRETE_STATE_CONSUMABLE_RESOURCE,
HOMOGENEOUS_AGGREGATED_CAPACITATED_RESOURCE,
HOMOGENEOUS_AGGREGATED_CAPACITATED_REUSABLE_RESOURCE,
HOMOGENEOUS_AGGREGATED_CAPACITATED_CONSUMABLE_RESOURCE,
HOMOGENEOUS_AGGREGATED_PERFORMER, HOMOGENEOUS_LABOR_TEAM,

HETEROGENEOUS_AGGREGATED_RESOURCE, 
HETEROGENEOUS_AGGREGATED_PERFORMER, HETEROGENEOUS_LABOR_TEAM,

SINGLE_RESOURCE=94,
SINGLE_DISCRETE_STATE_RESOURCE, SINGLE_DISCRETE_STATE_REUSABLE_RESOURCE, SINGLE_DISCRETE_STATE_CONSUMABLE_RESOURCE, 
SINGLE_CAPACITATED_RESOURCE, SINGLE_CAPACITATED_REUSABLE_RESOURCE, SINGLE_CAPACITATED_CONSUMABLE_RESOURCE=100, 

SINGLE_PERFORMER=101, SINGLE_WORKER, 
SINGLE_TOOL, 
SINGLE_STOCK,
OPERATED_OBJECT, OPERATED_LOCATION, OPERATED_POPULATION, OPERATED_RESOURCE, NON_SPECIAL_OPERATED_OBJECT=109,

MULTIPLE_RESOURCE_CAPACITY=112, RESOURCE_CAPACITY, INTEGER_RESOURCE_CAPACITY, FLOAT_RESOURCE_CAPACITY, STOCK_CAPACITY, INTEGER_STOCK_CAPACITY, FLOAT_STOCK_CAPACITY,
  
AVAILABILITY_CONSTRAINT=125, TIME_AVAILABILITY_CONSTRAINT, STATE_AVAILABILITY_CONSTRAINT, CAPACITY_AVAILABILITY_CONSTRAINT, STANDARD_CAPACITY_AVAILABILITY_CONSTRAINT,

AVAILABILITY_TIME_SLOT=130, RECURRENT_AVAILABILITY_TIMETABLE,

INCONSISTENCY_CONDITION, RESOURCE_SHARING_VIOLATION_CONDITION, ACTIVITY_INCONSISTENCY_CONDITION, ACTIVITIES_RESOURCES_INCONSISTENT_COMMITMENT, OCCURRENCE_DOMAIN, OCCURRENCE_CROSS_DOMAIN=137,

SETTLEMENT=141, MULTIPLE_SETTLEMENT, POPULATION, MULTIPLE_POPULATION, SIMPLE_POPULATION, SIMPLE_HOMOGENEOUS_POPULATION, SIMPLE_HETEROGENEOUS_POPULATION,

LOCATION=151,

SYSTEM=161, SET_SYSTEM, MANAGER, OPERATING_SYSTEM, CONTROLLED_SYSTEM, PRODUCTION_SYSTEM,

STRATEGY=171, ACTIVITIES_RESOURCES_BLOCK_SET, ACTIVITIES_RESOURCES_BLOCK,

GOAL=181, INT_GOAL, FLOAT_GOAL, INT_TARGET, FLOAT_TARGET, INT_ANTI_TARGET, FLOAT_ANTI_TARGET,

CRITERION=188, CRITERION_SET, GOAL_CRITERION, UTILITY_CRITERION, TARGET_UTILITY_CRITERION, ANTI_TARGET_UTILITY_CRITERION, GLOBAL_TARGET_CRITERION, GLOBAL_ANTI_TARGET_CRITERION
/* LAST=195*/
// cf. g_BD_ReservedUserClassId  ; mettre à jour g_BD_ReservedUserClassId
/* ne pas oublier d'ajuster C_MakeClassIdToCharTab dans C_CoreFunction.cc */
/* ne pas oublier d'ajuster C_InstallMetaClasses dans C_CoreFunction.cc */
};
//@}

//----------------------------------------------------------------------------
//      Symboles des Descripteurs 
//----------------------------------------------------------------------------

/**@name Symboles des classes de descripteurs

Enumération des descripteurs (sous-classes de [C]ConstantDescriptor[/C] et de [C]VariableDescriptor[/C]) définis dans \CDb et d'intérêt pour le développeur. Les descripteurs qui n'ont d'intérêt que pour le fonctionnement du moteur ne sont pas listés dans cette documentation.\\ \index{C_DescriptorId}\index{Descripteurs prédéfinis}

Ils sont tous exploités dans \CDb (dans les différentes sous-classes prédéfinies d'entités). Ils sont alors hérités par toutes les sous-classes dans les domaines d'application.\\

Aucun descripteur n'a été identifié de manière anticipatoire, pour être seulement exploités dans les sous-classes de [C]DescriptedEntity[/C] propres aux applications.\\

Avec chaque symbole, on indique ci-dessous la classe de descripteur qui le possède, l'entier associé et la ou les classes d'entités qui en sont dotées. C'est dans la page dédiée à la classe d'entités qu'est explicitée la sémantique du descripteur et qu'est indiqué le type de sa valeur.\\

{\large {\tt Constants :}}\\

\begin{tabular}{cccc}
{\tiny symbole de classe} & {\tiny classe de descripteur} & {\tiny entier associé} & {\tiny classe d'entité} \\ \hline
{\tiny ACCEPTABILITY_THRESHOLD} & {\tiny AcceptabilityThreshold} & {\tiny 21} & {\tiny OccurrenceDomain} \\

{\tiny ACTIVITY_LIST_ATTRIBUTE} & {\tiny ActivityListAttribute} & {\tiny 23} & {\tiny Goal} \\
{\tiny ACTIVITY_SET_DEFAULT_UTILITY } & {\tiny ActivitySetDefaultUtility } & {\tiny 24} & {\tiny Strategy} \\
{\tiny ACTIVITY_SET_SATISFACTION_THRESHOLD} & {\tiny ActivitySetSatisfactionThreshold} & {\tiny 25} & {\tiny Strategy} \\
{\tiny ALLOCATION_ATTRIBUTE} & {\tiny AllocationAttribute} & {\tiny 26} & {\tiny PrimitiveActivity SingleInstructionSet} \\
{\tiny ALLOCATION_PRIORITY_DEGREE} & {\tiny AllocationPriorityDegree} & {\tiny 27} & {\tiny PrimitiveActivity} \\
{\tiny ANTI_TARGET_WEIGHT } & {\tiny AntiTargetWeight } & {\tiny 28} & {\tiny Strategy} \\
{\tiny ARB_ATTRIBUTE} & {\tiny ArbAttribute & {\tiny 29} & {\tiny InstructionSetConjunction} \\
{\tiny ATTACHED_ENTITY} & {\tiny AttachedEntity} & {\tiny 30} & {\tiny ActivitiesResourcesBlock ControlledSystem TransferInformation} \\
{\tiny AVAILABILITY_MODE} & {\tiny AvailabilityMode} & {\tiny 31} & {\tiny AvailabilityConstraint RecurrentAvailabilityTimetable} \\
{\tiny AVAILABILITY_TIMETABLE_ATTRIBUTE} & {\tiny AvailabilityTimetableAttribute} & {\tiny 32} & {\tiny SingleResource} \\
{\tiny BINARY_OPERATOR_ID} & {\tiny BinaryOperatorId} & {\tiny 33} & {\tiny OccurrenceDomain} \\
{\tiny CAPACITY_AVAILABILITY_ATTRIBUTE} & {\tiny CapacityAvailabilityAttribute} & {\tiny 34} & {\tiny SingleCapacitatedResource HomogeneousAggregatedCapacitatedResource} \\
{\tiny CONTINUATION_MODE} & {\tiny ContinuationMode} & {\tiny 35} & {\tiny OperatedObjectSpecification OperationResourceSpecification PerformerSpecification} \\
{\tiny CONTROLLED} & {\tiny Controlled} & {\tiny 36} & {\tiny System} \\
{\tiny CRITERION_ATTRIBUTE} & {\tiny CriterionAttribute} & {\tiny 37} & {\tiny Goal} \\

{\tiny CRITERION_VALUE} & {\tiny CriterionValue} & {\tiny 39} & {\tiny Criterion} \\
{\tiny CURRENT_OPERATED_OBJECT} & {\tiny CurrentOperatedObject} & {\tiny 40} & {\tiny Operation} \\
{\tiny DESCRIBED_RESOURCE} & {\tiny DescribedResource & {\tiny 41} & {\tiny AvailabilityConstraint RecurrentAvailabilityTimetable ResourceCapacity MultipleResourceCapacity} \\

{\tiny DISTRIBUTION_MODE} & {\tiny DistributionMode} & {\tiny 43} & {\tiny TransferInformation} \\
{\tiny FLOAT_INF_CORE} & {\tiny FloatInfCore} & {\tiny 44} & {\tiny FloatGoal} \\
{\tiny FLOAT_INF_SUPPORT} & {\tiny FloatInfSupport} & {\tiny 45} & {\tiny FloatGoal} \\
{\tiny FLOAT_LOWER_LEVEL} & {\tiny FloatLowerLevel} & {\tiny 46} & {\tiny FloatStockCapacity} \\
{\tiny FLOAT_QUANTITY_SPEED} & {\tiny FloatQuantitySpeed} & {\tiny 47} & {\tiny FloatQuantityGranulatedOperation} \\
{\tiny FLOAT_SUP_CORE} & {\tiny FloatSupCore} & {\tiny 48} & {\tiny FloatGoal} \\
{\tiny FLOAT_SUP_SUPPORT} & {\tiny FloatSupSupport} & {\tiny 49} & {\tiny FloatGoal} \\
{\tiny FLOAT_UPPER_LEVEL} & {\tiny FloatUpperLevel} & {\tiny 50} & {\tiny FloatStockCapacity} \\
{\tiny GLOBAL_ANTI_TARGET_VALUE} & {\tiny GlobalAntiTargetValue} & {\tiny 51} & {\tiny InstructionSetConjunction} \\
{\tiny GLOBAL_TARGET_VALUE}      & {\tiny GlobalTargetValue}     & {\tiny 52} & {\tiny InstructionSetConjunction} \\
{\tiny GOAL_ATTRIBUTE} & {\tiny GoalAttribute} & {\tiny 53} & {\tiny GoalCriterion} \\

{\tiny INSTRUCTION_SET_DISJUNCTION_ATTRIBUTE} & {\tiny InstructionSetDisjunctionAttribute} & {\tiny 55} & {\tiny Activity} \\
{\tiny INT_INF_CORE} & {\tiny IntInfCore} & {\tiny 56} & {\tiny IntGoal} \\
{\tiny INT_INF_SUPPORT} & {\tiny IntInfSupport} & {\tiny 57} & {\tiny IntGoal} \\
{\tiny INT_LOWER_LEVEL} & {\tiny IntLowerLevel} & {\tiny 58} & {\tiny IntStockCapacity} \\
{\tiny INT_QUANTITY_SPEED} & {\tiny IntQuantitySpeed} & {\tiny 59} & {\tiny IntQuantityGranulatedOperation} \\
{\tiny INT_SUP_CORE} & {\tiny IntSupCore} & {\tiny 60} & {\tiny IntGoal} \\
{\tiny INT_SUP_SUPPORT} & {\tiny IntSupSupport} & {\tiny 61} & {\tiny IntGoal} \\
{\tiny INT_UPPER_LEVEL} & {\tiny IntUpperLevel} & {\tiny 62} & {\tiny IntStockCapacity} \\
{\tiny INVOLVED_INT_QUANTITY} & {\tiny InvolvedIntQuantity} & {\tiny 63} & {\tiny PrimitiveActivity} \\
{\tiny INVOLVED_FLOAT_QUANTITY} & {\tiny InvolvedFloatQuantity} & {\tiny 64} & {\tiny PrimitiveActivity} \\
{\tiny IS_SINGLE} & {\tiny IsSingle} & {\tiny 65} & {\tiny Resource} \\

{\tiny IS_DISCRETE_STATE} & {\tiny IsDiscreteState} & {\tiny 67} & {\tiny SingleResource} \\
{\tiny IS_PERMANENT} & {\tiny IsPermanent} & {\tiny 68} & {\tiny Resource} \\
{\tiny IS_REUSABLE} & {\tiny IsReusable} & {\tiny 69} & {\tiny SingleResource} \\
{\tiny IS_ONE_SHOT} & {\tiny IsOneShot} & {\tiny 70} & {\tiny ActivityOptional} \\


{\tiny OBJECT_ENTITY_SPEC_ATTRIBUTE} & {\tiny ObjectEntitySpecAttribute} & {\tiny 73} & {\tiny Goal OperatedObjectSpecification} \\
{\tiny OBJECT_SET_TYPE} & {\tiny ObjectSetType} & {\tiny 74} & {\tiny InstructionSet} \\
{\tiny OCCURRENCE_CROSS_DOMAIN_ATTRIBUTE} & {\tiny OccurrenceCrossDomainAttribute} & {\tiny 75} & {\tiny InconsistencyCondition} \\
{\tiny OPERATED_ENTITY_ID} & {\tiny OperatedEntityId} & {\tiny 76} & {\tiny Operation} \\

{\tiny OPERATION_SPEC_CONTAINER } & {\tiny OperationSpecContainer} & {\tiny 78} & {\tiny Operation} \\
{\tiny OPERATION_USED} & {\tiny OperationUsed} & {\tiny 79} & {\tiny OperationUsageItem} \\
{\tiny PERFORMER_ENTITY_SPEC_ATTRIBUTE} & {\tiny PerformerEntitySpecAttribute} & {\tiny 80} & {\tiny PerformerSpecification} \\
{\tiny PRIMITIVE_ACTIVITY_ATTRIBUTE} & {\tiny PrimitiveActivityAttribute} & {\tiny 81} & {\tiny SingleInstructionSet} \\
{\tiny PRIORITY_DEGREE} & {\tiny PriorityDegree} & {\tiny 82} & {\tiny Operation} \\
{\tiny PROGRESSION_MODE} & {\tiny ProgressionMode} & {\tiny 83} & {\tiny Operation} \\
{\tiny PROPORTIONALITY_MODE} & {\tiny ProportionalityMode} & {\tiny 84} & {\tiny OperationResourceSpecification} \\
{\tiny QUANTITY_DESCRIPTOR_ID} & {\tiny QuantityDescriptorId} & {\tiny 85} & {\tiny IntQuantityGranulatedOperation FloatQuantityGranulatedOperation} \\
{\tiny RESOURCE_CAPACITY_DEMAND} & {\tiny ResourceCapacityDemand} & {\tiny 86} & {\tiny HomogeneousAggregatedCapacitatedResource SingleCapacitatedResource} \\
{\tiny RESOURCE_CAPACITY_SUPPLY} & {\tiny ResourceCapacitySupply} & {\tiny 87} & {\tiny HomogeneousAggregatedCapacitatedResource SingleCapacitatedResource} \\
{\tiny RESOURCE_CONSTRAINT_LIST} & {\tiny ResourceConstraintList} & {\tiny 88} & {\tiny OperatingSystem} \\
{\tiny RESOURCE_ENTITY} & {\tiny ResourceEntity} & {\tiny 89} & {\tiny OperatedObject} \\
{\tiny RESOURCE_ENTITY_MEMORY} & {\tiny ResourceEntityMemory} & {\tiny 90} & {\tiny OperatedObject} \\
{\tiny RESOURCE_ENTITY_SPEC_ATTRIBUTE} & {\tiny ResourceEntitySpecAttribute} & {\tiny 91} & {\tiny OperationResourceSpecification} \\
{\tiny RESOURCE_EXCLUSION_LIST_ATTRIBUTE} & {\tiny ResourceExclusionListAttribute} & {\tiny 92} & {\tiny SingleInstructionSet} \\
{\tiny RESOURCE_LIST} & {\tiny ResourceList} & {\tiny 93} & {\tiny CoengagementItem} \\

{\tiny RESOURCE_USED} & {\tiny ResourceUsed} & {\tiny 95} & {\tiny ResourceUsageItem} \\
{\tiny SELECTOR_FCT_ARG} & {\tiny SelectorFctArg} & {\tiny 96} & {\tiny OperatedObjectSpecification OperationResourceSpecification PerformerSpecification} \\
{\tiny SELECTOR_FCT_ID} & {\tiny SelectorFctId} & {\tiny 97} & {\tiny OperatedObjectSpecification OperationResourceSpecification PerformerSpecification} \\
{\tiny SET_ALLOCATION_ATTRIBUTE} & {\tiny SetAllocationAttribute} & {\tiny 98} & {\tiny InstructionSetConjunction} \\
{\tiny SET_ALLOCATIONS_ATTRIBUTE} & {\tiny SetAllocationsAttribute} & {\tiny 99} & {\tiny InstructionSetConjunction} \\
{\tiny STATE_AVAILABILITY_ATTRIBUTE} & {\tiny StateAvailabilityAttribute} & {\tiny 100} & {\tiny SingleResource} \\
{\tiny STATE_DOMAIN} & {\tiny StateDomain} & {\tiny 101} & {\tiny StateAvailabilityConstraint} \\
{\tiny STEP} & {\tiny Step} & {\tiny 102} & {\tiny Operation} \\
{\tiny SUSPENSION_THRESHOLD} & {\tiny SuspensionThreshold} & {\tiny 103} & {\tiny Operation} \\
{\tiny TARGET_WEIGHT} & {\tiny TargetWeight} & {\tiny 104} & {\tiny Strategy} \\
{\tiny TIME_AVAILABILITY_ATTRIBUTE} & {\tiny TimeAvailabilityAttribute} & {\tiny 105} & {\tiny SingleResource} \\
{\tiny TIME_SPEED} & {\tiny TimeSpeed} & {\tiny 106} & {\tiny TimeGranulatedOperation} \\
{\tiny TRANSFER_QUANTITY_ID} & {\tiny TransferQuantityId} & {\tiny 107} & {\tiny TransferInformation} \\
{\tiny TRANSFER_QUANTITY_INT_VALUE} & {\tiny TransferQuantityIntValue} & {\tiny 108} & {\tiny TransferInformation} \\
{\tiny TRANSFER_QUANTITY_OPTION} & {\tiny TransferQuantityOption} & {\tiny 109} & {\tiny TransferInformation} \\

{\tiny TRANSFER_QUANTITY_FLOAT_VALUE} & {\tiny TransferQuantityFloatValue} & {\tiny 111} & {\tiny TransferInformation} \\

{\tiny UNIT_SPEED} & {\tiny UnitSpeed} & {\tiny 113} & {\tiny UnitGranulatedOperation} \\
{\tiny VALUE_FOR_SELECTION} & {\tiny ValueForSelection} & {\tiny 114} & {\tiny -} \\
\end{tabular}

{\large {\tt Variables :}}\\

\begin{tabular}{cccc}
{\tiny symbole de classe} & {\tiny classe de descripteur} & {\tiny entier associé} & {\tiny classe d'entité} \\ \hline
{\tiny ACHIEVEMENT_DEGREE} & {\tiny AchievementDegree} & {\tiny 151} & {\tiny Operation} \\
{\tiny ACTIVITY_ACTION_MODE} & {\tiny ActivityActionMode} & {\tiny 152} & {\tiny Goal} \\
{\tiny ACTIVITY_CLOSING_THRESHOLD} & {\tiny ActivityClosingThreshold} & {\tiny 153} & {\tiny Operation} \\
{\tiny ACTIVITY_CONTAINER_LIST} & {\tiny ActivityContainerList} & {\tiny 154} & {\tiny OperatedObjectSpecification OperationSpecification PerformerSpecification} \\
{\tiny ALLOCATED_OPERATION} & {\tiny AllocatedOperation} & {\tiny 155} & {\tiny PrimitiveActivity} \\
{\tiny AVAILABILITY_STATUS} & {\tiny AvailabilityStatus} & {\tiny 156} & {\tiny Resource} \\
{\tiny BEG_DATE} & {\tiny BegDate} & {\tiny 157} & {\tiny Activity Operation AvailabilityTimeSlot} \\
{\tiny CLOSE_TO_CLOSE_DELAYS} & {\tiny CloseToCloseDelays} & {\tiny 158} & {\tiny ActivityOverlapping ActivityInclusion} \\
{\tiny CLOSE_TO_OPEN_DELAYS} & {\tiny CloseToOpenDelays} & {\tiny 159} & {\tiny ActivityBefore ActivityBeforeAny ActivityIteration} \\

{\tiny CONSTRAINED_ENTITY_CLASS_ID} & {\tiny ConstrainedEntityClassId} & {\tiny 161} & {\tiny InconsistencyCondition} \\
{\tiny DESTINATION_SPEC_ATTRIBUTE} & {\tiny DestinationSpecAttribute} & {\tiny 162} & {\tiny TransferActivity} \\
{\tiny DEVELOPPED_LIST} & {\tiny DeveloppedList} & {\tiny 163} & {\tiny OperatedObjectSpecification OperationSpecification PerformerSpecification} \\
{\tiny DURATION} & {\tiny Duration} & {\tiny 164} & {\tiny Operation} \\
{\tiny DYNAMIC_ACTIVITY_TO_CONTINUE} & {\tiny DynamicActivityToContinue} & {\tiny 165} & {\tiny Activity} \\
{\tiny END_DATE} & {\tiny EndDate} & {\tiny 166} & {\tiny Activity Operation AvailabilityTimeSlot} \\
{\tiny ENTITY_SPEC_REFERENCE_CLASS_ID} & {\tiny EntitySpecReferenceClassId} & {\tiny 167} & {\tiny OperatedObjectSpecification OperationResourceSpecification OperationSpecification PerformerSpecification} \\
{\tiny ENTITY_SPEC_REFERENCE_CLASS_NAME} & {\tiny EntitySpecReferenceClassName} & {\tiny 168} & {\tiny OperatedObjectSpecification OperationResourceSpecification OperationSpecification PerformerSpecification} \\
{\tiny ENTITY_SPEC_REFERENT_ENTITY} & {\tiny EntitySpecReferentEntity} & {\tiny 169} & {\tiny OperatedObjectSpecification} \\
{\tiny FLOAT_CURRENT_LEVEL} & {\tiny FloatCurrentLevel} & {\tiny 170} & {\tiny FloatStockCapacity} \\
{\tiny FLOAT_IN_CAPACITY} & {\tiny FloatInCapacity} & {\tiny 171} & {\tiny FloatStockCapacity} \\
{\tiny FLOAT_OUT_CAPACITY} & {\tiny FloatOutCapacity} & {\tiny 172} & {\tiny FloatStockCapacity} \\
{\tiny FULLY_OPERATED_BY} & {\tiny FullyOperatedBy} & {\tiny 173} & {\tiny OperatedObject} \\
{\tiny GOAL_LIST} & {\tiny GoalList} & {\tiny 174} & {\tiny PrimitiveActivity} \\
{\tiny INSTRUCTION_CONTAINER} & {\tiny InstructionContainer} & {\tiny 175} & {\tiny Operation} \\
{\tiny INSTRUCTION_SET_CONSISTENCY} & {\tiny InstructionSetConsistency} & {\tiny 176} & {\tiny InstructionSetConjunction} \\
{\tiny INSTRUCTION_SET_MAXIMALITY} & {\tiny InstructionSetMaximality} & {\tiny 177} & {\tiny InstructionSetConjunction} \\
{\tiny INT_CURRENT_LEVEL} & {\tiny IntCurrentLevel} & {\tiny 178} & {\tiny IntStockCapacity} \\
{\tiny INT_IN_CAPACITY} & {\tiny IntInCapacity} & {\tiny 179} & {\tiny IntStockCapacity} \\
{\tiny INT_OUT_CAPACITY} & {\tiny IntOutCapacity} & {\tiny 180} & {\tiny IntStockCapacity} \\
{\tiny IS_ALLOCATED} & {\tiny IsAllocated} & {\tiny 181} & {\tiny SingleInstructionSet InstructionSetConjunction} \\




{\tiny IS_OPENING_PREDICATE_NOT_FALSE} & {\tiny IsOpeningPredicateNotFalse} & {\tiny 186} & {\tiny Activity} \ \

{\tiny LAST_RESTARTING_DATE} & {\tiny LastRestartingDate} & {\tiny 188} & {\tiny Operation} \\



{\tiny LIST_TO_OPERATE} & {\tiny ListToOperate} & {\tiny 192} & {\tiny OperatedObjectSpecification} \\
{\tiny MAX_BEG_DATE} & {\tiny MaxBegDate} & {\tiny 193} & {\tiny Activity} \\
{\tiny MAX_END_DATE} & {\tiny MaxEndDate} & {\tiny 194} & {\tiny Activity} \\
{\tiny MAX_NUMBER_OF_REPLICATIONS} & {\tiny MaxNumberOfReplications} & {\tiny 195} & {\tiny ActivityIteration} \\
{\tiny MEMORY_SCOPE} & {\tiny MemoryScope} & {\tiny 196} & {\tiny Manager} \\
{\tiny MIN_BEG_DATE} & {\tiny MinBegDate} & {\tiny 197} & {\tiny Activity} \\
{\tiny MIN_END_DATE} & {\tiny MinEndDate} & {\tiny 198} & {\tiny Activity} \\
{\tiny MIN_NUMBER_OF_REPLICATIONS} & {\tiny MinNumberOfReplications} & {\tiny 199} & {\tiny ActivityIteration} \\
{\tiny NB_INVOLVED_ENTITIES} & {\tiny NbInvolvedEntities} & {\tiny 200} & {\tiny PrimitiveActivity} \\
{\tiny NET_DURATION} & {\tiny NetDuration} & {\tiny 201} & {\tiny NetDuration} \\
{\tiny NON_OPERATED_ENTITY_LIST} & {\tiny NonOperatedEntityList} & {\tiny 202} & {\tiny OperatedObject} \\
{\tiny NON_OPERATED_PROPORTION} & {\tiny NonOperatedProportion} & {\tiny 203} & {\tiny OperatedObject} \\
{\tiny NON_OPERATED_INT_QUANTITY} & {\tiny NonOperatedIntQuantity} & {\tiny 204} & {\tiny OperatedObject} \\
{\tiny NON_OPERATED_FLOAT_QUANTITY} & {\tiny NonOperatedFloatQuantity} & {\tiny 205} & {\tiny OperatedObject} \\
{\tiny OPEN_TO_CLOSE_DELAYS} & {\tiny OpenToCloseDelays} & {\tiny 206} & {\tiny ActivityOverlapping } \\
{\tiny OPEN_TO_OPEN_DELAYS} & {\tiny OpenToOpenDelays} & {\tiny 207} & {\tiny ActivityBefore ActivityBeforeAny ActivityOverlapping ActivityInclusion ActivityIteration} \\
{\tiny OPERATED_ENTITY_LIST} & {\tiny OperatedEntityList} & {\tiny 208} & {\tiny OperatedObject} \\
{\tiny OPERATED_OBJECT_SPEC_ATTRIBUTE} & {\tiny OperatedObjectSpecAttribute} & {\tiny 209} & {\tiny ActivityPrimitive} \\
{\tiny OPERATED_PROPORTION} & {\tiny OperatedProportion} & {\tiny 210} & {\tiny OperatedObject} \\
{\tiny OPERATED_INT_QUANTITY} & {\tiny OperatedIntQuantity} & {\tiny 211} & {\tiny OperatedObject} \\
{\tiny OPERATED_FLOAT_QUANTITY} & {\tiny OperatedFloatQuantity} & {\tiny 212} & {\tiny OperatedObject} \\
{\tiny OPERATION_CLASS_ID} & {\tiny OperationClassId} & {\tiny 213} & {\tiny OperationSpecification PrimitiveActivity} \\
{\tiny OPERATION_CLASS_NAME} & {\tiny OperationClassName} & {\tiny 214} & {\tiny OperationSpecification PrimitiveActivity} \\
{\tiny OPERATION_SPEC_ATTRIBUTE} & {\tiny OperationSpecAttribute} & {\tiny 215} & {\tiny PrimitiveActivity} \\
{\tiny PERFORMER_SPEC_ATTRIBUTE} & {\tiny PerformerSpecAttribute} & {\tiny 216} & {\tiny PrimitiveActivity} \\
{\tiny POWER} & {\tiny Power} & {\tiny 217} & {\tiny SinglePerformer HomogeneousAggregatedPerformer HeterogeneousAggregatedPerformer} \\
{\tiny PRE_EXPANDED_LIST} & {\tiny PreExpandedList} & {\tiny 218} & {\tiny OperatedObjectSpecification OperationResourceSpecification PerformerSpecification} \\
{\tiny PREDICTION_SCOPE} & {\tiny PredictionScope} & {\tiny 219} & {\tiny Manager} \\
{\tiny READY_TO_RUN} & {\tiny ReadyToRun} & {\tiny 220} & {\tiny Activity} \\
{\tiny REPLICATION_COUNTER} & {\tiny ReplicationCounter} & {\tiny 221} & {\tiny ActivityIteration} \\


{\tiny REQUIRED_RESOURCES} & {\tiny RequiredResources} & {\tiny 224} & {\tiny Operation OperationSpecification} \\
{\tiny RESOURCE_ID} & {\tiny ResourceId} & {\tiny 225} & {\tiny CoengagementItem} \\
{\tiny ROOT_TAB} & {\tiny RootTab} & {\tiny 2256 & {\tiny Activity} \\
{\tiny SITUATION} & {\tiny Situation} & {\tiny 227} & {\tiny Activity Operation SingleInstructionSet} \\
{\tiny SPECIFIED_OPERATION} & {\tiny SpecifiedOperation} & {\tiny 228} & {\tiny OperationSpecification} \\
{\tiny TRANSFER_INFORMATION_ATTRIBUTE} & {\tiny TransferInformationAttribute} & {\tiny 229} & {\tiny TransferActivity} \\

{\tiny WEIGHT_ATTRIBUTE} & {\tiny WeightAttribute} & {\tiny 231} & {\tiny Goal Criterion CriterionSet}
\end{tabular}
*/
//@{

//cf g_BD_ReservedUserDescriptorId ; mettre à jour g_BD_ReservedUserDescriptorId
enum C_DescriptorId {
  // constants
ACCEPTABILITY_THRESHOLD=21, 
ACHIEVEMENT_DEGREE_INCREMENT=22,  // + JPR 090806
ACTIVITY_LIST_ATTRIBUTE,
ACTIVITY_SET_DEFAULT_UTILITY, 
ACTIVITY_SET_SATISFACTION_THRESHOLD,
ALLOCATION_ATTRIBUTE,
ALLOCATION_PRIORITY_DEGREE,  // + JPR 100608
ANTI_TARGET_WEIGHT, 
ARB_ATTRIBUTE,
ATTACHED_ENTITY,
AVAILABILITY_MODE=31,
AVAILABILITY_TIMETABLE_ATTRIBUTE, // + JPR 100520
BINARY_OPERATOR_ID,
CAPACITY_AVAILABILITY_ATTRIBUTE,
CONTINUATION_MODE,
CONTROLLED,
CRITERION_ATTRIBUTE,
CRITERION_SET_SUB_NODES,
CRITERION_VALUE,
CURRENT_OPERATED_OBJECT,  // + JPR 100804
DESCRIBED_RESOURCE=41,
DEVELOPMENT_FAILURE,
DISTRIBUTION_MODE,
FLOAT_INF_CORE,
FLOAT_INF_SUPPORT,
FLOAT_LOWER_LEVEL,
FLOAT_QUANTITY_SPEED,
FLOAT_SUP_CORE,
FLOAT_SUP_SUPPORT,
FLOAT_UPPER_LEVEL,
GLOBAL_ANTI_TARGET_VALUE=51,
GLOBAL_TARGET_VALUE,
GOAL_ATTRIBUTE,
HAS_SITUATION_CHANGED,
INSTRUCTION_SET_DISJUNCTION_ATTRIBUTE,  // + JPR 061120
INT_INF_CORE,
INT_INF_SUPPORT,
INT_LOWER_LEVEL,
INT_QUANTITY_SPEED,
INT_SUP_CORE,
INT_SUP_SUPPORT=61,
INT_UPPER_LEVEL,
INVOLVED_INT_QUANTITY,
INVOLVED_FLOAT_QUANTITY,
IS_SINGLE,
IS_DESTINATION,
IS_DISCRETE_STATE,
IS_PERMANENT,  // + JPR 110225
IS_REUSABLE,
IS_ONE_SHOT,  // + JPR 071031
MAX_BEG_STATE_REACHED=71,  // + JPR 111122
NO_OPERATION_SPEC,        // contraire à l'ontologie (JPR 120112)
OBJECT_ENTITY_SPEC_ATTRIBUTE,
OBJECT_SET_TYPE,
OCCURRENCE_CROSS_DOMAIN_ATTRIBUTE,
OPERATED_ENTITY_ID,
OPERATED_PACK_ATTRIBUTE,
OPERATION_SPEC_CONTAINER ,
OPERATION_USED,
PERFORMER_ENTITY_SPEC_ATTRIBUTE,
PRIMITIVE_ACTIVITY_ATTRIBUTE=81,
PRIORITY_DEGREE,
PROGRESSION_MODE,
PROPORTIONALITY_MODE,
QUANTITY_DESCRIPTOR_ID,
RESOURCE_CAPACITY_DEMAND,
RESOURCE_CAPACITY_SUPPLY,
RESOURCE_CONSTRAINT_LIST,
RESOURCE_ENTITY,
RESOURCE_ENTITY_MEMORY,
RESOURCE_ENTITY_SPEC_ATTRIBUTE=91,
RESOURCE_EXCLUSION_LIST_ATTRIBUTE,  // + JPR 100505
RESOURCE_LIST,
RESOURCE_USAGE,
RESOURCE_USED,
SELECTOR_FCT_ARG,
SELECTOR_FCT_ID,
SET_ALLOCATION_ATTRIBUTE,
SET_ALLOCATIONS_ATTRIBUTE,
STATE_AVAILABILITY_ATTRIBUTE,
STATE_DOMAIN=101,
STEP,
SUSPENSION_THRESHOLD,
TARGET_WEIGHT,
TIME_AVAILABILITY_ATTRIBUTE,
TIME_SPEED,
TRANSFER_QUANTITY_ID,
TRANSFER_QUANTITY_INT_VALUE,
TRANSFER_QUANTITY_OPTION,
TRANSFER_REM_QUANTITY_INT_VALUE,
TRANSFER_QUANTITY_FLOAT_VALUE=111,
TRANSFER_REM_QUANTITY_FLOAT_VALUE,
UNIT_SPEED,
VALUE_FOR_SELECTION,
VIOLATED_CONSTRAINT_TAB_ATTRIBUTE=115,         // + JPR 100504
/* ne pas oublier d'ajuster C_MakeDescIdToCharTab dans C_CoreFunction.cc */
/* ne pas oublier d'ajuster C_InstallMetaClasses dans C_CoreFunction.cc */
  // variables
ACHIEVEMENT_DEGREE=151,
ACTIVITY_ACTION_MODE,
ACTIVITY_CLOSING_THRESHOLD,
ACTIVITY_CONTAINER_LIST,
ALLOCATED_OPERATION,
AVAILABILITY_STATUS,
BEG_DATE,
CLOSE_TO_CLOSE_DELAYS,
CLOSE_TO_OPEN_DELAYS,
CLOSED_YET_VALIDATED,
CONSTRAINED_ENTITY_CLASS_ID=161,
DESTINATION_SPEC_ATTRIBUTE,
DEVELOPPED_LIST,
DURATION,
DYNAMIC_ACTIVITY_TO_CONTINUE,
END_DATE,
ENTITY_SPEC_REFERENCE_CLASS_ID,
ENTITY_SPEC_REFERENCE_CLASS_NAME,
ENTITY_SPEC_REFERENT_ENTITY,
FLOAT_CURRENT_LEVEL,
FLOAT_IN_CAPACITY=171,
FLOAT_OUT_CAPACITY,
FULLY_OPERATED_BY,
GOAL_LIST,
INSTRUCTION_CONTAINER,
INSTRUCTION_SET_CONSISTENCY,
INSTRUCTION_SET_MAXIMALITY,
INT_CURRENT_LEVEL,
INT_IN_CAPACITY,
INT_OUT_CAPACITY,
IS_ALLOCATED=181,
IS_CLOSED_VALID,
IS_OPEN_VALID,
IS_WAITING_VALID,
IS_OPENING_PREDICATE_CHECKED,
IS_OPENING_PREDICATE_NOT_FALSE,
LAST_PROCEED_OPERATION_DATE,
LAST_RESTARTING_DATE,
LAST_VALIDATION_DIRECTION,
LAST_VALIDATION_ORIGINE,
LAST_VALIDATION_REQUEST=191,
LIST_TO_OPERATE,
MAX_BEG_DATE,
MAX_END_DATE,
MAX_NUMBER_OF_REPLICATIONS,
MEMORY_SCOPE,
MIN_BEG_DATE,
MIN_END_DATE,
MIN_NUMBER_OF_REPLICATIONS,
NB_INVOLVED_ENTITIES,
NET_DURATION=201,
NON_OPERATED_ENTITY_LIST,
NON_OPERATED_PROPORTION,
NON_OPERATED_INT_QUANTITY,
NON_OPERATED_FLOAT_QUANTITY,
OPEN_TO_CLOSE_DELAYS,
OPEN_TO_OPEN_DELAYS,
OPERATED_ENTITY_LIST,
OPERATED_OBJECT_SPEC_ATTRIBUTE,
OPERATED_PROPORTION,
OPERATED_INT_QUANTITY=211,
OPERATED_FLOAT_QUANTITY,
OPERATION_CLASS_ID,
OPERATION_CLASS_NAME,
OPERATION_SPEC_ATTRIBUTE,
PERFORMER_SPEC_ATTRIBUTE,
POWER,
PRE_EXPANDED_LIST,
PREDICTION_SCOPE,
READY_TO_RUN,
REPLICATION_COUNTER=221,
  //REPLICATION_MAX_BEG_DATE=222,
  //REPLICATION_MIN_BEG_DATE,
REQUIRED_RESOURCES=224,
RESOURCE_ID,
ROOT_TAB,
SITUATION,
SPECIFIED_OPERATION,
TRANSFER_INFORMATION_ATTRIBUTE,
VALIDATION_FROM,
WEIGHT_ATTRIBUTE=231
/* LAST=230*/
/* ne pas oublier d'ajuster C_MakeDescIdToCharTab dans C_CoreFunction.cc */
/* ne pas oublier d'ajuster C_InstallMetaClasses dans C_CoreFunction.cc */
//cf g_BD_ReservedUserDescriptorId ; mettre à jour g_BD_ReservedUserDescriptorId
};
//@}


//----------------------------------------------------------------------------
//      Symboles des Méthodes
//----------------------------------------------------------------------------

/**@name Symboles des classes des méthodes

Enumération des méthodes (sous-classes de [C]Method[/C]) définies dans \CDb et d'intérêt pour le développeur. Les méthodes qui n'ont d'intérêt que pour le fonctionnement du moteur ne sont pas listés dans cette documentation.\\ \index{C_MethodId}\index{Méthodes prédéfinies}

Elles sont toutes exploitées dans \CDb (dans les différentes sous-classes prédéfinies d'entités). Elles sont alors héritées par toutes les sous-classes dans les domaines d'application.\\

Aucune méthode n'a été identifiée de manière anticipatoire, pour être seulement exploitées dans les sous-classes de [C]Entity[/C] propres aux applications.\\

Avec chaque symbole, on indique ci-dessous la classe de méthode qui le possède, l'entier associé et la ou les classes d'entités qui en sont dotées. C'est dans la page dédiée à la classe d'entités qu'est explicité le rôle de la méthode et qu'est indiqué le type et la sémantique de sa valeur retournée par le corps.\\

\begin{tabular}{cccc}
{\tiny symbole de classe} & {\tiny classe de méthode} & {\tiny entier associé} & {\tiny classe d'entité} \\ \hline
{\tiny OPENING_PREDICATE} & {\tiny OpeningPredicate} & {\tiny 101} & {\tiny Activity} \\
{\tiny CLOSING_PREDICATE} & {\tiny ClosingPredicate} & {\tiny 102} & {\tiny Activity} \\
{\tiny MIN_BEG_STATE_PREDICATE} & {\tiny MinBegStatePredicate} & {\tiny 103} & {\tiny Activity} \\
{\tiny MAX_BEG_STATE_PREDICATE} & {\tiny MaxBegStatePredicate} & {\tiny 104} & {\tiny Activity} \\
{\tiny SET_USER_OPERATION_NAME} & {\tiny SetUserOperationName} & {\tiny 105} & {\tiny PrimitiveActivity} \\
{\tiny UPDATE_REACTIVATED_SON} & {\tiny UpdateReactivatedSon} & {\tiny 106} & {\tiny ActivityIteration} \\
{\tiny MODIFY_IN_OPEN_SITUATION} & {\tiny ModifyInOpenSituation} & {\tiny 107} & {\tiny ActivityBefore ActivityBeforeAny ActivityMeeting ActivityMeetingAny ActivityOverlapping ActivityInclusion ActivityCostarting ActivityCoending ActivityEquality ActivityConjunction ActivityDisjunction} \\

{\tiny STATE_TRANSITION_PROCEDURE} & {\tiny StateTransitionProcedure} & {\tiny 115} & {\tiny Operation} \\
{\tiny FEASIBILITY_CONDITION} & {\tiny FeasibilityCondition} & {\tiny 116} & {\tiny Operation} \\
{\tiny SET_USER_UNIT_SPEED} & {\tiny SetUserUnitSpeed} & {\tiny 117} & {\tiny UnitGranulatedOperation} \\
{\tiny SET_USER_INT_QUANTITY_SPEED} & {\tiny SetUserIntQuantitySpeed} & {\tiny 118} & {\tiny IntQuantityGranulatedOperation} \\
{\tiny SET_USER_FLOAT_QUANTITY_SPEED} & {\tiny SetUserFloatQuantitySpeed} & {\tiny 119} & {\tiny FloatQuantityGranulatedOperation} \\
{\tiny SET_USER_TIME_SPEED} & {\tiny SetUserTimeSpeed} & {\tiny 120} & {\tiny TimeGranulatedOperation} \\
{\tiny SET_USER_STEP} & {\tiny SetUserStep} & {\tiny 121} & {\tiny Operation} \\

{\tiny SET_USER_TRANSFERED_INT_QUANTITY} & {\tiny SetUserTransferedIntQuantity} & {\tiny 125} & {\tiny TransferInformation} \\
{\tiny SET_USER_TRANSFERED_FLOAT_QUANTITY} & {\tiny SetUserTransferedFloatQuantity} & {\tiny 126} & {\tiny TransferInformation} \\

{\tiny INVOLVED_ENTITIES} & {\tiny InvolvedEntities} & {\tiny 128} & {\tiny OperatedObject} \\

{\tiny STATE_RELATED_PREDICATE} & {\tiny StateRelatedPredicate} & {\tiny 130} & {\tiny StateAvailabilityConstraint} \\
{\tiny CAPACITY_OFFER_DEMAND_COMPARISON} & {\tiny CapacityOfferDemandComparison} & {\tiny 131} & {\tiny CapacityAvailabilityConstraint} \\
{\tiny HOLDING_CONDITION} & {\tiny HoldingCondition} & {\tiny 132} & {\tiny InconsistencyCondition} \\
{\tiny SET_THIS_TIME_SLOT_BEG_VALUE} & {\tiny SetThisTimeSlotBegValue} & {\tiny 133} & {\tiny AvailabilityTimeSlot} \\
{\tiny SET_THIS_TIME_SLOT_END_VALUE} & {\tiny SetThisTimeSlotEndValue} & {\tiny 134} & {\tiny AvailabilityTimeSlot} \\
{\tiny SET_NEXT_TIME_SLOT_BEG_VALUE} & {\tiny SetNextTimeSlotBegValue} & {\tiny 135} & {\tiny AvailabilityTimeSlot} \\

{\tiny GET_CRITERION_VALUE} & {\tiny GetCriterionValue} & {\tiny 138} & {\tiny Criterion} \\

{\tiny INT_FUTURE_VALUE_BY_ACTIVITY} & {\tiny IntFutureValueByActivity} & {\tiny 140} & {\tiny IntGoal} \\
{\tiny FLOAT_FUTURE_VALUE_BY_ACTIVITY} & {\tiny FloatFutureValueByActivity} & {\tiny 141} & {\tiny FloatGoal} \\
{\tiny INT_FUTURE_VALUE_BY_ACTIVITY_LIST} & {\tiny IntFutureValueByActivityList} & {\tiny 142} & {\tiny IntGoal} \\
{\tiny FLOAT_FUTURE_VALUE_BY_ACTIVITY_LIST} & {\tiny FloatFutureValueByActivityList} & {\tiny 143} & {\tiny FloatGoal} \\

{\tiny BEST_ACTIVITY_SET_SELECTOR} & {\tiny BestActivitySetSelector} & {\tiny 145} & {\tiny Manager} \\
{\tiny LOCAL_BEST_ACTIVITY_SET_SELECTOR} & {\tiny LocalBestActivitySetSelector} & {\tiny 146} & {\tiny ActivitiesResourcesBlock} \\
{\tiny CANDIDATE_ACTIVITY_SET_SORT_METHOD} & {\tiny CandidateActivitySetSortMethod} & {\tiny 147} & {\tiny Manager} \\
{\tiny LOCAL_CANDIDATE_ACTIVITY_SET_SORT_METHOD} & {\tiny LocalCandidateActivitySetSortMethod} & {\tiny 148} & {\tiny ActivitiesResourcesBlock} \\
\end{tabular}
*/
// cf. g_BD_ReservedUserMethodId ; mettre à jour g_BD_ReservedUserMethodId
// !!!! METTRE À JOUR C_MakeMethodIdToCharTab !!!!
// !!!! METTRE À JOUR g_BD_ReservedUserMethodId (C_CoreFunction.cc)
//@{
enum C_CoreMethodId {
OPENING_PREDICATE=101,
CLOSING_PREDICATE,
MIN_BEG_STATE_PREDICATE,
MAX_BEG_STATE_PREDICATE,
SET_USER_OPERATION_NAME,             // + JPR 110111
UPDATE_REACTIVATED_SON,
MODIFY_IN_OPEN_SITUATION,
UPDATE_PREV_WHEN_SON_OPEN,
UPDATE_NEXT_WHEN_SON_OPEN,
UPDATE_PREV_WHEN_SON_CLOSED,
UPDATE_NEXT_WHEN_SON_CLOSED=111,

STATE_TRANSITION_PROCEDURE=115,
FEASIBILITY_CONDITION,
SET_USER_UNIT_SPEED,
SET_USER_INT_QUANTITY_SPEED,
SET_USER_FLOAT_QUANTITY_SPEED,
SET_USER_TIME_SPEED,
SET_USER_STEP=121,

SET_USER_TRANSFERED_INT_QUANTITY=125,
SET_USER_TRANSFERED_FLOAT_QUANTITY=126,

INVOLVED_ENTITIES=128,

STATE_RELATED_PREDICATE=130,
CAPACITY_OFFER_DEMAND_COMPARISON=131,
HOLDING_CONDITION,
SET_THIS_TIME_SLOT_BEG_VALUE,
SET_THIS_TIME_SLOT_END_VALUE,
SET_NEXT_TIME_SLOT_BEG_VALUE=135,

GET_CRITERION_VALUE=138,
UTILITY_BY_ACTIVITY_LIST,  // unused
INT_FUTURE_VALUE_BY_ACTIVITY,
FLOAT_FUTURE_VALUE_BY_ACTIVITY,
INT_FUTURE_VALUE_BY_ACTIVITY_LIST,
FLOAT_FUTURE_VALUE_BY_ACTIVITY_LIST=143,

BEST_ACTIVITY_SET_SELECTOR=145,
LOCAL_BEST_ACTIVITY_SET_SELECTOR,
CANDIDATE_ACTIVITY_SET_SORT_METHOD,
LOCAL_CANDIDATE_ACTIVITY_SET_SORT_METHOD=148

/* LAST=160*/
// !!!! METTRE À JOUR g_BD_ReservedUserMethodId (C_CoreFunction.cc)
// !!!! METTRE À JOUR C_MakeMethodIdToCharTab !!!!
};
//@}


//----------------------------------------------------------------------------
//      Symboles des Arguments
//----------------------------------------------------------------------------

/**@name Symboles des classes des arguments

Enumération des arguments de méthode (sous-classes de [C]Argument[/C]) définies dans \CDb et d'intérêt pour le développeur. Les arguments qui n'ont d'intérêt que pour le fonctionnement du moteur ne sont pas listés dans cette documentation.\\ \index{C_ArgumentId}\index{Arguments prédéfinis}

Ils sont tous exploités dans \CDb (dans les différentes sous-classes prédéfinies de méthodes). Ils sont alors hérités par toutes les sous-classes dans les domaines d'application.\\

Aucun argument n'a été identifié de manière anticipatoire, pour être seulement exploitées dans les sous-classes de [C]Method[/C] propres aux applications.\\

Avec chaque symbole, on indique ci-dessous la classe d'arguments qui le possède, l'entier associé et la ou les classes de méthodes qui en sont dotées. C'est dans la page dédiée à la classe de méthodes qu'est explicité le rôle de l'argument et qu'est indiqué le type et la sémantique de sa valeur envoyée au corps de la méthode.\\


\begin{tabular}{cccc}
{\tiny OPERATION_TARGET} & {\tiny OperationTarget} & {\tiny 51} & {\tiny StateTransitionProcedure} \\
{\tiny TRANSFER_OPERATION_SOURCE} & {\tiny TransferOperationSource} & {\tiny 52} & {\tiny StateTransitionProcedure} \\
{\tiny TRANSFER_OPERATION_DESTINATION} & {\tiny TransferOperationDestination} & {\tiny 53} & {\tiny StateTransitionProcedure} \\
{\tiny OPEN_SON_RANK} & {\tiny OpenSonRank} & {\tiny 54} & {\tiny UpdateNextWhenSonOpen UpdatePrevWhenSonOpen} \\
{\tiny CLOSED_SON_RANK} & {\tiny ClosedSonRank} & {\tiny 55} & {\tiny UpdateNextWhenSonClosed UpdatePrevWhenSonClosed} \\

{\tiny SINGLE_ACTIVITY_SET_ARGUMENT} & {\tiny SingleActivitySetArgument} & {\tiny 72} & {\tiny IntFutureValueByActivity FloatFutureValueByActivity} \\ 
{\tiny SINGLE_LIST_ARGUMENT} & {\tiny SingleListArgument} & {\tiny 73} & {\tiny IntFutureValueByActivityList FloatFutureValueByActivityList} \\  
{\tiny TIME_DOMAIN_ARGUMENT} & {\tiny TimeDomainArgument} & {\tiny 76} & {\tiny IntFutureValueByActivity FloatFutureValueByActivity IntFutureValueByActivityList FloatFutureValueByActivityList} \\

{\tiny ACTIVITIES_RESOURCES_BLOCK_ARGUMENT} & {\tiny ActivitiesResourcesBlockArgument} & {\tiny 81} & {\tiny BestActivitySetSelector CandidateActivitySetSortMethod} \\ 
{\tiny ACTIVITY_SETS_ARGUMENT} & {\tiny ActivitySetsArgument} & {\tiny 82} & {\tiny BestActivitySetSelector LocalBestActivitySetSelector} \\
{\tiny ACTIVITY_SET_ARGUMENT} & {\tiny ActivitySetArgument} & {\tiny 83 & {\tiny CandidateActivitySetSortMethod LocalCandidateActivitySetSortMethod}
\end{tabular}

*/
// cf. g_BD_ReservedUserArgumentId ; mettre à jour g_BD_ReservedUserArgumentId
//@{
enum C_CoreArgumentId {
OPERATION_TARGET=51, 
TRANSFER_OPERATION_SOURCE, 
TRANSFER_OPERATION_DESTINATION,
OPEN_SON_RANK, CLOSED_SON_RANK,

SINGLE_ACTIVITY_SET_ARGUMENT=72, 
SINGLE_LIST_ARGUMENT=73,

TIME_DOMAIN_ARGUMENT=76,

ACTIVITIES_RESOURCES_BLOCK_ARGUMENT=81, 
ACTIVITY_SETS_ARGUMENT, 
ACTIVITY_SET_ARGUMENT=83 
/* LAST=90*/
// cf. g_BD_ReservedUserArgumentId ; mettre à jour g_BD_ReservedUserArgumentId
// METTRE à JOUR C_MakeArgumentIdToCharTab

/*
IN_OUT_ACCESS_MODE=61, // obsolète
QUANTITY_ID, // obsolète 
QUANTITY_OPTION, // obsolète 
QUANTITY_INTEGER_VALUE, // obsolète  
QUANTITY_FLOAT_VALUE=65, // obsolète 

ACTIVITY_ARGUMENT=71, // obsolète   
ACTIVITY_LIST_ARGUMENT=74, // obsolète   
RESOURCE_LIST_ARGUMENT=75, // obsolète 
 */
};
//@}  // fin argId

//@}  // fin symboles classes prédéfinies

//@}  // fin Constantes symboliques (enum)

//@}  // fin Types prédéfinis ... constantes ...

         // tableaux de conversion enum -> chaine de caractères
extern TableauString StateIdToCharTab;   // + JPR 0510
extern TableauString ContinuationModeToCharTab;
extern TableauString DistributionModeToCharTab;
extern TableauString ProportionalityModeToCharTab;
extern TableauString SelectionModeToCharTab;
extern TableauString StockAccesModeToCharTab;
extern TableauString QuantityIdToCharTab;
extern TableauString QuantityOptionToCharTab;
extern TableauString GoalIdToCharTab;
extern TableauString ActivityActionToCharTab;

         // / Une variable dont la valeur est un treillis d'ensembles de critères.
extern CriterionSet* gWeightLattice;
         // / Une autre variable dont la valeur est un treillis d'ensembles de critères.
extern CriterionSet* gFinalWeightLattice;  

#endif  // _C_CORETYPES_H
